#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

# file      : build.py
# project   : basebuildc
# copyright : (C) 2014 by Claude BULIN
#
# This file is part of basebuildc project
#
# basebuildc is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# basebuildc is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA


import sys, os
try:
	from gi.repository import Gtk, GObject, Gdk
except:
	print("GTK Not Availible")
	sys.exit(1)
from datetime import date
import datetime
import subprocess, shutil, stat
from fconfig import FConfig
import Language_Init
from prefs import Preferences
from wind_languages import wind_languages
from code_gen import codegen_py


class build_write():

	def __init__( self,
					project_name='',
					project_version='',
					project_author='',
					pathdest="",
					list_po=[],
					bool_manpage=False,
					bool_pixmaps=False,
					name_lib_statique="",
					name_menu="",
					name_mime="",
					bool_gtk3=False,
					bool_glib=False,
					list_files_glade=[],
					list_src=[] ):

		self.list_source_file = []

		self.project_name       = project_name
		self.project_version    = project_version
		self.project_author     = project_author
		self.pathdest           = pathdest
		self.ListUpdateLangs    = list_po
		self.bool_manpage       = bool_manpage
		self.bool_pixmaps       = bool_pixmaps
		self.name_lib_statique  = name_lib_statique
		self.name_menu          = name_menu
		self.name_mime          = name_mime
		self.bool_gtk3          = bool_gtk3
		self.bool_glib          = bool_glib
		self.ListNameFilesGlade = list_files_glade
		self.ListScrFolders     = list_src

		print("build():__init__")
		print("self.project_name        = ", self.project_name )
		print("self.project_version     = ", self.project_version )
		print("self.project_author      = ", self.project_author )
		print("self.pathdest            = ", self.pathdest )
		print("self.ListUpdateLangs     = ", self.ListUpdateLangs )
		print("self.bool_manpage        = ", self.bool_manpage )
		print("self.bool_pixmaps        = ", self.bool_pixmaps )
		print("self.name_lib_statique   = ", self.name_lib_statique )
		print("self.name_menu           = ", self.name_menu )
		print("self.name_mime           = ", self.name_mime )
		print("self.bool_gtk3           = ", self.bool_gtk3 )
		print("self.bool_glib           = ", self.bool_glib )
		print("self.ListNameFilesGlade  = ", self.ListNameFilesGlade )
		print("self.ListScrFolders      = ", self.ListScrFolders )

	def write_entete(self, fp, p_name_func='NAME_FUNCTION'):
		fp.write( '/*\n' )
		fp.write( ' *  file      : %s\n' % p_name_func )
		fp.write( ' *  project   : %s\n' % self.project_name )
		fp.write( ' *  copyright : (C) %s by %s\n' % (date.today().year, self.project_author) )
		fp.write( ' * \n' )
		fp.write( ' *  This file is part of %s project\n' % self.project_name )
		fp.write( ' * \n' )
		fp.write( ' *  %s is free software; you can redistribute it and/or\n' % self.project_name )
		fp.write( ' *  modify it under the terms of the GNU Lesser General Public\n' )
		fp.write( ' *  License as published by the Free Software Foundation; version 3.\n' )
		fp.write( ' * \n' )
		fp.write( ' *  %s is distributed in the hope that it will be useful,\n' % self.project_name )
		fp.write( ' *  but WITHOUT ANY WARRANTY; without even the implied warranty of\n' )
		fp.write( ' *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n' )
		fp.write( ' *  Lesser General Public License for more details.\n' )
		fp.write( ' * \n' )
		fp.write( ' *  You should have received a copy of the GNU Lesser General Public\n' )
		fp.write( ' *  License along with this library; if not, write to the Free Software\n' )
		fp.write( ' *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307\n' )
		fp.write( ' *  USA\n' )
		fp.write( ' */\n' )
		fp.write( '\n\n' )

	def write_makefile_am(self):
		print( 'write_makefile_am' )
		fp = open( self.pathdest + "/Makefile.am", "w" )
		fp.write( '#\n# Makefile.am with Base-Build-C\n#\n' )
		fp.write( 'SUBDIRS = ' )
		if self.ListUpdateLangs:
			fp.write( 'po ' )
		if self.name_lib_statique != "":
			fp.write( '%s '  % self.name_lib_statique )
		for src in self.ListScrFolders:
			fp.write( '%s '  % src[0] )
		if self.bool_manpage:
			fp.write( 'man ' )
		if self.bool_pixmaps:
			fp.write( 'pixmaps ' )
		if self.name_menu != "":
			fp.write( 'menu ' )
		if self.name_mime != "":
			fp.write( 'mime ' )
		if self.ListNameFilesGlade:
			fp.write( 'glade ' )
		fp.write( '\nACLOCAL_AMFLAGS = -I m4\n' )
		fp.write( 'EXTRA_DIST = ' )
		if self.ListUpdateLangs:
			fp.write( 'config.rpath m4/ChangeLog ' )
		fp.write( 'autogen.sh\n' )
		fp.close()

	def write_lib_statique(self):
		print( 'write_lib_statique' )
		if self.name_lib_statique == "":
			return
		os.makedirs( self.pathdest + "/" + self.name_lib_statique )
		#
		# Makefile.am
		#
		fp = open( self.pathdest + "/" + self.name_lib_statique + "/Makefile.am", "w" )
		fp.write( '#\n# %s/Makefile.am with BaseBuild\n#\n' % self.name_lib_statique )
		fp.write( 'AUTOMAKE_OPTIONS=gnu\n' )
		if self.bool_gtk3 or self.bool_glib:
			fp.write( '\nAM_CPPFLAGS = \\\n' )
			fp.write( '   -DPACKAGE_DATA_DIR=\\\"\"$(datadir)\"\\\" \\\n' )
			fp.write( '   -DPACKAGE_LOCALE_DIR=\\\"\"$(prefix)/$(DATADIRNAME)/locale\"\\\" \\\n' )
			fp.write( '   -DPIXMAPS_DIR=\\\"\"$(datadir)/pixmaps/%s\"\\\" \\\n' % self.project_name )
			fp.write( '	@PACKAGE_CFLAGS@\n' )
			fp.write( '\nAM_CFLAGS = @PACKAGE_CFLAGS@ \\\n' )
			fp.write( '   -DDATA_DIR=\\\"\"$(datadir)\"\\\" \\\n' )
			fp.write( '   -DLOCALE_DIR=\\\"\"$(datadir)/locale\"\\\" \\\n' )
			fp.write( '   -DPIXMAPS_DIR=\\\"\"$(datadir)/pixmaps/%s\"\\\" \n' % self.project_name )
		fp.write( '\nnoinst_LIBRARIES=%s.a\n' % self.name_lib_statique )
		fp.write( '\n%s_a_SOURCES=lib.c xdg.c lib.h\n' % self.name_lib_statique )
		fp.close()
		#
		# lib.h
		#
		fp = open( self.pathdest + "/" + self.name_lib_statique + "/lib.h", "w" )
# 		fp.write( '/*\n' )
# 		fp.write( ' *  %s/lib.h\n' % self.name_lib_statique )
# 		fp.write( ' */\n' )
# 		fp.write( '\n' )
		self.write_entete( fp, p_name_func='%s/lib.h' % self.name_lib_statique )
		fp.write( '#ifndef __libH_h__\n' )
		fp.write( '#define __libH_h__ 1\n' )
		fp.write( '\n' )
		fp.write( 'void\tlibH_print_mess( char *p_mess );\n' )
		fp.write( '\n' )
		fp.write( '// XDG.C\n' )
		fp.write( '// \n' )
		fp.write( 'void		xdg_print_list_config_path( void );\n' )
		fp.write( 'void		xdg_print_list_data_path( void );\n' )
		fp.write( 'gchar		*xdg_search_config_xdg( void );\n' )
		fp.write( 'gchar		*xdg_search_data_xdg( gchar *NameFile );\n' )
		fp.write( '\n' )
		fp.write( '#endif\n' )
		fp.write( '\n' )
		fp.close()
		#
		# lib.c
		#
		# append to po/POTFILES.in
		self.list_source_file.append( '%s/lib.c' % self.name_lib_statique )
		fp = open( self.pathdest + "/" + self.name_lib_statique + "/lib.c", "w" )
# 		fp.write( '/*\n' )
# 		fp.write( ' *  %s/lib.c\n' % self.name_lib_statique )
# 		fp.write( ' */\n' )
# 		fp.write( '\n' )
		self.write_entete( fp, p_name_func='%s/lib.c' % self.name_lib_statique )
		fp.write( '#ifdef HAVE_CONFIG_H\n' )
		fp.write( '	#include \"../config.h\"\n' )
		fp.write( '#endif\n' )
		fp.write( '\n' )
		fp.write( '#ifdef ENABLE_NLS\n' )
		fp.write( '	#include <libintl.h>\n' )
		fp.write( '	#define _(String) gettext (String)\n' )
		fp.write( '#endif\n' )
		fp.write( '\n' )
		if self.bool_gtk3:
			fp.write( '#include <gtk/gtk.h>\n' )
		if self.bool_glib:
			fp.write( '#include <glib.h>\n' )
			fp.write( '\n' )
		fp.write( '#include <stdio.h>\n' )
		fp.write( '#include \"lib.h\"\n' )
		fp.write( '\n' )
		fp.write( 'void libH_print_mess( char *p_mess )\n' )
		fp.write( '{\n' )
		fp.write( '\tprintf( \"\%s\", p_mess );\n' )
		fp.write( '}\n' )
		fp.write( '\n' )
		fp.close()
		#
		# xdg.c
		#
		GstrSrc = \
		'\n' \
		'\n' \
		'\n' \
		'// ------------------------------------------------------------------------\n' \
		'// ************************************************************************\n' \
		'// \n' \
		'// XDG_DATA_HOME\n' \
		'// HOME\n' \
		'// XDG_DATA_DIRS\n' \
		'// \n' \
		'// ************************************************************************\n' \
		'// ------------------------------------------------------------------------\n' \
		'\n' \
		'// \n' \
		'// \n' \
		'GSList *xdg_remove_gslist( GSList *gslist )\n' \
		'{\n' \
		'	GSList *Lthemesemp = gslist;\n' \
		'	GSList *Lcur;\n' \
		'\n' \
		'	for (Lcur = Lthemesemp; Lcur && Lcur->data; Lcur = Lcur->next) {\n' \
		'		g_free( Lcur->data );\n' \
		'		Lcur->data = NULL;\n' \
		'	}\n' \
		'	g_slist_free(Lthemesemp);\n' \
		'	return( (GSList *)NULL );\n' \
		'}\n' \
		'\n' \
		'// \n' \
		'// FROM GRAVEMAN\n' \
		'// \n' \
		'// on regarde si un fichier de configuration existe \n' \
		'// tel que definie par http://standards.freedesktop.org/basedir-spec/latest/\n' \
		'// \n' \
		'// construction de la liste des PATH dans lequel on va chercher la configuration\n' \
		'// \n' \
		'GSList *xdg_get_data_path( void )\n' \
		'{\n' \
		'	GSList	*Lnewlist = NULL;\n' \
		'	const	gchar *Lenv;\n' \
		'	gchar	*Lpath;\n' \
		'	gchar	**Llistpath;\n' \
		'	gint	i;\n' \
		'\n' \
		'	if ((Lenv=g_getenv(\"XDG_DATA_HOME\"))) {\n' \
		'		Lpath = g_strdup_printf(\"%s/%s\", Lenv, PACKAGE);\n' \
		'		Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	} else if ((Lenv=g_getenv(\"HOME\"))) {\n' \
		'		Lpath = g_strdup_printf(\"%s/.local/share/%s\", Lenv, PACKAGE);\n' \
		'		Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	}\n' \
		'\n' \
		'	if ((Lenv = g_getenv(\"XDG_DATA_DIRS\"))) {\n' \
		'		Llistpath = g_strsplit(Lenv, \":\", 0);\n' \
		'\n' \
		'		for (i=0; Llistpath[i]; i++) {\n' \
		'			Lpath = g_strdup_printf(\"%s/%s\", Llistpath[i], PACKAGE);\n' \
		'			Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'		}\n' \
		'\n' \
		'		g_strfreev(Llistpath);\n' \
		'	}\n' \
		'\n' \
		'	Lpath = g_strdup_printf(\"/usr/local/share/%s\", PACKAGE);\n' \
		'	Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	Lpath = g_strdup_printf(\"/usr/share/%s\", PACKAGE);\n' \
		'	Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	Lpath = g_strdup_printf(\"/usr/share/%s/glade\", PACKAGE);\n' \
		'	Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	Lpath = g_strdup_printf(\"%s/%s\", DATA_DIR, PACKAGE);\n' \
		'	Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	Lpath = g_strdup(\"/usr/share/pixmaps\");\n' \
		'	Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	Lpath = g_strdup_printf(\"/usr/share/pixmaps/%s\", PACKAGE);\n' \
		'	Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'\n' \
		'	return Lnewlist;\n' \
		'}\n' \
		'// \n' \
		'// \n' \
		'gchar *xdg_search_data_xdg( gchar *NameFile )\n' \
		'{	\n' \
		'	GSList		*Lthemesemp = xdg_get_data_path();\n' \
		'	GSList		*Lcur;\n' \
		'	gchar		*Name = NULL;\n' \
		'	gboolean	BoolExist = FALSE;\n' \
		'\n' \
		'	for (Lcur = Lthemesemp; Lcur && Lcur->data; Lcur = Lcur->next) {\n' \
		'		Name = g_strdup_printf( \"%s/%s\", (gchar *)Lcur->data, NameFile );\n' \
		'		BoolExist = g_file_test( Name, G_FILE_TEST_IS_REGULAR );\n' \
		'		if( TRUE == BoolExist ) break;\n' \
		'		g_free( Name );\n' \
		'		Name = NULL;\n' \
		'	}\n' \
		'	Lthemesemp = xdg_remove_gslist( Lthemesemp );\n' \
		'	return( (gchar *)Name );\n' \
		'}\n' \
		'// \n' \
		'// \n' \
		'void xdg_print_list_data_path( void )\n' \
		'{\n' \
		'	// if( TRUE == OptionsCommandLine.BoolVerboseMode ) {\n' \
		'\n' \
		'		GSList	*Lthemesemp = xdg_get_data_path();\n' \
		'		GSList *Lcur;\n' \
		'\n' \
		'		g_print( \"\\n\" );\n' \
		'		g_print( \"%s :: %s (line = %d)\\n\", __FILE__, __FUNCTION__, __LINE__ );\n' \
		'		g_print( \"!---------------------------------------------\\n\" );\n' \
		'		g_print( \"! FreeDesktop integration\\n\" );\n' \
		'		g_print( \"!---------------------------------------------\\n\" );\n' \
		'		g_print( \"! http://www.freedesktop.org/wiki/\\n\" );\n' \
		'		g_print( \"! http://fr.wikipedia.org/wiki/Freedesktop.org\\n\" );\n' \
		'		g_print( \"!---------------------------------------------\\n\" );\n' \
		'		for (Lcur = Lthemesemp; Lcur && Lcur->data; Lcur = Lcur->next) {\n' \
		'			g_print( \"\tLcur->data = %s\\n\", (gchar *)Lcur->data );\n' \
		'		}\n' \
		'\n' \
		'		Lthemesemp = xdg_remove_gslist( Lthemesemp );\n' \
		'	//}\n' \
		'}\n' \
		'\n' \
		'// \n' \
		'// \n' \
		'GSList *xdg_get_config_path( void )\n' \
		'{\n' \
		'	GSList	*Lnewlist = NULL;\n' \
		'	const	gchar *Lenv;\n' \
		'	gchar	*Lpath;\n' \
		'	gchar	**Llistpath;\n' \
		'	gint	i;\n' \
		'\n' \
		'	// if( TRUE == OptionsCommandLine.BoolVerboseMode ) {\n' \
		'	// 	g_print( "\\n" );\n' \
		'	// 	g_print ("%s :: %s (line = %d)\\n", __FILE__, __FUNCTION__, __LINE__);\n' \
		'	// }\n' \
		'	if ((Lenv=g_getenv("XDG_CONFIG_HOME"))) {\n' \
		'		Lpath = g_strdup_printf("%s/%s/%s.conf", Lenv, PACKAGE, PACKAGE);\n' \
		'		Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	} else if ((Lenv=g_getenv("HOME"))) {\n' \
		'		Lpath = g_strdup_printf("%s/.config/%s/%s.conf", Lenv, PACKAGE, PACKAGE);\n' \
		'		Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	}\n' \
		'\n' \
		'	if ((Lenv = g_getenv("XDG_CONFIG_DIRS"))) {\n' \
		'		Llistpath = g_strsplit(Lenv, ":", 0);\n' \
		'\n' \
		'		for (i=0; Llistpath[i]; i++) {\n' \
		'			if (!*Llistpath[i]) continue; // on ignore les entrees vides\n' \
		'			Lpath = g_strdup_printf("%s/%s/%s.conf", Llistpath[i], PACKAGE, PACKAGE);\n' \
		'			Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'			// if( TRUE == OptionsCommandLine.BoolVerboseMode )\n' \
		'			// 	g_print( "Lpath = %s\\n", Lpath );\n' \
		'		}\n' \
		'\n' \
		'		g_strfreev(Llistpath);\n' \
		'	} else {\n' \
		'		Lpath = g_strdup_printf("/etc/xdg/%s/%s.conf", PACKAGE, PACKAGE);\n' \
		'		Lnewlist = g_slist_append(Lnewlist, Lpath);\n' \
		'	}\n' \
		'\n' \
		'	return Lnewlist;\n' \
		'}\n' \
		'// \n' \
		'// \n' \
		'gchar *xdg_search_config_xdg( void )\n' \
		'{	\n' \
		'	GSList		*Lthemesemp = xdg_get_config_path();\n' \
		'	GSList		*Lcur;\n' \
		'	gchar		*Name = NULL;\n' \
		'	gboolean	BoolExist = FALSE;\n' \
		'\n' \
		'	// if( TRUE == OptionsCommandLine.BoolVerboseMode ) {\n' \
		'	// 	g_print( "\\n" );\n' \
		'	// 	g_print ("%s :: %s (line = %d)\\n", __FILE__, __FUNCTION__, __LINE__);\n' \
		'	// }\n' \
		'	for (Lcur = Lthemesemp; Lcur && Lcur->data; Lcur = Lcur->next) {\n' \
		'		Name = g_strdup( (gchar *)Lcur->data );\n' \
		'		// BoolExist = libutils_test_file_exist( Name );\n' \
		'		BoolExist = g_file_test( Name, G_FILE_TEST_IS_REGULAR );\n' \
		'		// if( TRUE == OptionsCommandLine.BoolVerboseMode )\n' \
		'		// 	g_print( "\\t%s  NameSearch = %s\\n", BoolExist ? "TRUE " : "FALSE", Name );\n' \
		'		if( TRUE == BoolExist ) break;\n' \
		'		g_free( Name );\n' \
		'		Name = NULL;\n' \
		'	}\n' \
		'	// SI NameFile N\'EXISTE PAS ALORS SELECTION DE LA PREMIERE ENTREE\n' \
		'	if( FALSE == BoolExist ) {\n' \
		'		Lcur = Lthemesemp;\n' \
		'		Name = g_strdup( (gchar *)Lcur->data );\n' \
		'	}\n' \
		'	// if( TRUE == OptionsCommandLine.BoolVerboseMode )\n' \
		'	// 	g_print( "\\n" );\n' \
		'	Lthemesemp = xdg_remove_gslist( Lthemesemp );\n' \
		'	return( (gchar *)Name );\n' \
		'}\n' \
		'//\n' \
		'//\n' \
		'void xdg_print_list_config_path( void )\n' \
		'{\n' \
		'	// if( TRUE == OptionsCommandLine.BoolVerboseMode ) {\n' \
		'\n' \
		'		GSList	*Lcur;\n' \
		'		GSList	*Lthemesemp = xdg_get_config_path();\n' \
		'\n' \
		'		g_print( "\\n" );\n' \
		'		g_print( "%s :: %s (line = %d)\\n", __FILE__, __FUNCTION__, __LINE__);\n' \
		'		g_print( "!---------------------------------------------\\n" );\n' \
		'		g_print( "! FreeDesktop integration\\n" );\n' \
		'		g_print( "!---------------------------------------------\\n" );\n' \
		'		g_print( "! http://www.freedesktop.org/wiki/\\n" );\n' \
		'		g_print( "! http://fr.wikipedia.org/wiki/Freedesktop.org\\n" );\n' \
		'		g_print( "!---------------------------------------------\\n" );\n' \
		'		for (Lcur = Lthemesemp; Lcur && Lcur->data; Lcur = Lcur->next) {\n' \
		'			g_print( "\tLcur->data = %s\\n", (gchar *)Lcur->data );\n' \
		'		}\n' \
		'\n' \
		'		Lthemesemp = xdg_remove_gslist( Lthemesemp );\n' \
		'	// }\n' \
		'}\n' \
		'\n'









		# append to po/POTFILES.in
		self.list_source_file.append( '%s/xdg.c' % self.name_lib_statique )
		fp = open( self.pathdest + "/" + self.name_lib_statique + "/xdg.c", "w" )
# 		fp.write( '/*\n' )
# 		fp.write( ' *  %s/xdg.c\n' % self.name_lib_statique )
# 		fp.write( ' */\n' )
# 		fp.write( '\n' )
		self.write_entete( fp, p_name_func='%s/xdg.c' % self.name_lib_statique )
		fp.write( '#ifdef HAVE_CONFIG_H\n' )
		fp.write( '	#include \"../config.h\"\n' )
		fp.write( '#endif\n' )
		fp.write( '\n' )
		fp.write( '#ifdef ENABLE_NLS\n' )
		fp.write( '	#include <libintl.h>\n' )
		fp.write( '	#define _(String) gettext (String)\n' )
		fp.write( '#endif\n' )
		fp.write( '\n' )
		fp.write( '#include <glib.h>\n' )
		fp.write( '\n' )
		fp.write( '#include <stdio.h>\n' )
		fp.write( '#include <stdlib.h>\n' )
		fp.write( '#include <string.h>\n' )
		fp.write( '\n' )
		fp.write( '#include "lib.h"\n' )
		fp.write( GstrSrc )
		fp.close()

	def write_glade_makefile_am(self):
		print( 'write_glade_makefile_am' )
		if not self.ListNameFilesGlade:
			return
		os.makedirs( self.pathdest + "/glade"  )
		fp = open( self.pathdest + "/glade/Makefile.am", "w" )
		fp.write( '#\n# glade/Makefile.am with BaseBuild\n#\n' )
		fp.write( 'AUTOMAKE_OPTIONS=gnu\n' )
		fp.write( 'glade_DATA = ' )
		for glade in self.ListNameFilesGlade:
			fp.write( '%s ' % os.path.basename( glade ) )
		fp.write( '\n' )
		fp.write( 'gladedir = $(datadir)/%s\n' % self.project_name )
		fp.write( 'EXTRA_DIST = $(glade_DATA)\n' )
		fp.write( '\nuninstall:\n' )
		fp.write( '\trm -rf \"$(DESTDIR)$(datadir)/%s/\";\n' % self.project_name )
		fp.write( '\n' )
		fp.close()

	def write_folder_src(self):
		print( 'write_folder_src' )
		for src in self.ListScrFolders:
			os.makedirs( self.pathdest + "/" + src[0] )
			if src[0] == "src":
				#
				# create: global.h
				#
				fp = open( self.pathdest + "/" + src[0] + "/global.h", "w" )
# 				fp.write( '/*\n' )
# 				fp.write( ' *  global.h\n' )
# 				fp.write( ' */\n' )
# 				fp.write( '\n' )
				self.write_entete( fp, p_name_func='%s/global.h' % src[0] )
				fp.write( '#ifndef global_h\n' )
				fp.write( '#define global_h 1\n' )
				fp.write( '\n' )
				if self.ListNameFilesGlade:
					fp.write( 'GtkBuilder *Builder_open( gchar *p_name_glade, GtkBuilder *p_GtkBuilder );\n' )
				fp.write( '\n' )
				for glade in self.ListNameFilesGlade:
					BaseName = os.path.basename( glade )
					Name = BaseName.split(".")[0]
					Ext  = BaseName.split(".")[-1]
					fp.write( 'gboolean %s_init( GtkWidget *FromWindMain );\n' % Name )
				fp.write( '\n' )
				fp.write( '\n' )
				fp.write( '#endif\n' )
				fp.close()
				#
				# create: main.c
				#
				# append to po/POTFILES.in
				self.list_source_file.append( 'src/main.c' )
				fp = open( self.pathdest + "/" + src[0] + "/main.c", "w" )
# 				fp.write( '/*\n' )
# 				fp.write( ' *  main.c\n' )
# 				fp.write( ' */\n' )
# 				fp.write( '\n' )
				self.write_entete( fp, p_name_func='%s/main.c' % src[0] )
				fp.write( '#ifdef HAVE_CONFIG_H\n' )
				fp.write( '	#include \"../config.h\"\n' )
				fp.write( '#endif\n' )
				fp.write( '\n' )
				fp.write( '#ifdef ENABLE_NLS\n' )
				fp.write( '	#include <libintl.h>\n' )
				fp.write( '	#define _(String) gettext (String)\n' )
				fp.write( '#endif\n' )
				fp.write( '\n' )
				if self.bool_gtk3:
					fp.write( '#include <gtk/gtk.h>\n' )
				if self.bool_glib:
					fp.write( '#include <glib.h>\n' )
				fp.write( '\n' )
				fp.write( '#ifdef ENABLE_STATIC_LIBRARY\n' )
				fp.write( '	#include "../%s/lib.h"\n' % self.name_lib_statique )
				fp.write( '#endif\n' )
				fp.write( '\n' )
				if self.ListUpdateLangs:
					fp.write( '#include <locale.h>\n' )
				fp.write( '#include <stdlib.h>\n' )
				fp.write( '#include <stdio.h>\n' )
				fp.write( '#include \"global.h\"\n' )
				fp.write( '\n' )
				fp.write( '\n' )
				if self.bool_glib:
					fp.write( '/* gint main (gint argc, gchar *argv[], gchar **envp)\n' )
				else:
					fp.write( '/* int main (int argc, char *argv[], char **envp)\n' )
				fp.write( ' *\n' )
				fp.write( ' * argc  designe le nombre d''arguments transmis au moment du lancement de l''executable\n' )
				fp.write( ' * argv  designe le vecteur contenant les differents arguments\n' )
				fp.write( ' * envp  designe le vecteur contenant les informations sur l''environnement\n' )
				fp.write( ' */\n' )
				if self.bool_glib:
					fp.write( 'gint main( gint argc, gchar *argv[], gchar **envp )\n' )
				else:
					fp.write( 'int main( int argc, char *argv[], char **envp )\n' )
				fp.write( '{\n' )
				fp.write( '\n' )
				fp.write( '\tprintf( "%s :: %s (line = %d)\\n", __FILE__, __FUNCTION__, __LINE__ );\n' )
				fp.write( '\tprintf( "\\n" );\n' )
				fp.write( '\tprintf( "***************************\\n" );\n' )
				fp.write( '\tprintf( "%s %s\\n", PACKAGE, VERSION );\n' )
				fp.write( '\tprintf( "***************************\\n" );\n' )
				fp.write( '\tprintf( "%s %s compiled at %s %s\\n", PACKAGE, VERSION, __DATE__, __TIME__ );\n' )
				fp.write( '\tprintf( "Compiler: gcc %s\\n", __VERSION__ );\n' )
				fp.write( '\tprintf( "***************************\\n" );\n' )
				if self.ListNameFilesGlade:
					fp.write( '\tg_print( "Gtk version  = %d.%d.%d\\n", GTK_MAJOR_VERSION, GTK_MICRO_VERSION, GTK_MINOR_VERSION );\n' )
					fp.write( '\tg_print( "Glib version = %d.%d.%d\\n", GLIB_MAJOR_VERSION, GLIB_MICRO_VERSION, GLIB_MINOR_VERSION );\n' )
					fp.write( '\tg_print( "***************************\\n\\n" );\n' )
				fp.write( '#ifdef ENABLE_NLS\n' )
				if self.bool_gtk3 == False:
					fp.write( '#define LOCALE_DIR "/usr/share/locale"\n' )
				fp.write( '\tprintf( "ENABLE_NLS OK\\n" );\n' )
				fp.write( '\tprintf( "\tGETTEXT_PACKAGE    = %s\\n", GETTEXT_PACKAGE );\n' )
				fp.write( '\tprintf( "\tLOCALE_DIR         = %s\\n", LOCALE_DIR );\n' )
				fp.write( '\tsetlocale( LC_ALL, "" );\n' )
				fp.write( '\tbindtextdomain( GETTEXT_PACKAGE, LOCALE_DIR );\n' )
				fp.write( '\tbind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );\n' )
				fp.write( '\ttextdomain( GETTEXT_PACKAGE );\n' )
				fp.write( '#endif\n' )
				if self.ListNameFilesGlade:
					fp.write( '\tgtk_init( &argc, &argv );\n' )
				fp.write( '\n' )
				if self.ListUpdateLangs:
					for glade in self.ListNameFilesGlade:
						BaseName = os.path.basename( glade )
						Name = BaseName.split(".")[0]
						Ext  = BaseName.split(".")[-1]
						fp.write( '\t%s_init( NULL );\n' % Name )
					if self.ListUpdateLangs:
						fp.write( '\tprintf( _("Hello world !\\n") );\n' )
					else:
						fp.write( '\tprintf( "Hello world !\\n" );\n' )
					fp.write( '\n' )
				if self.bool_gtk3 and  self.ListNameFilesGlade:
					fp.write( '\tgtk_main();\n' )
				fp.write( '\n' )
				fp.write( '\texit( EXIT_SUCCESS );\n' )
				fp.write( '}\n\n' )
				fp.close()
				#
				# create: builder.c
				#
				if self.ListNameFilesGlade:
					# append to po/POTFILES.in
					self.list_source_file.append( 'src/builder.c' )
					fp = open( self.pathdest + "/" + src[0] + "/builder.c", "w" )
# 					fp.write( '/*\n' )
# 					fp.write( ' *  builder.c\n' )
# 					fp.write( ' */\n' )
# 					fp.write( '\n' )
					self.write_entete( fp, p_name_func='%s/builder.c' % src[0] )
					fp.write( '#ifdef HAVE_CONFIG_H\n' )
					fp.write( '	#include "../config.h"\n' )
					fp.write( '#endif\n' )
					fp.write( '#include <gtk/gtk.h>\n' )
					fp.write( '#include <string.h>\n' )
					fp.write( '\n' )
					fp.write( '#ifdef ENABLE_STATIC_LIBRARY\n' )
					fp.write( '	#include "../%s/lib.h"\n' % self.name_lib_statique )
					fp.write( '#endif\n' )
					fp.write( '\n' )
					fp.write( '#include "global.h"\n' )
					# fp.write( '#include "builder.h"\n' )
					fp.write( '\n' )
					fp.write( 'GtkBuilder *Builder_open( gchar *p_name_glade, GtkBuilder *p_GtkBuilder )\n' )
					fp.write( '{\n' )
					fp.write( '	GError			*p_err = NULL;\n' )
					fp.write( '	gchar			*CurrentDir = g_get_current_dir ();\n' )
					fp.write( '	gchar			*PathCurrentDir = NULL;\n' )
					fp.write( '	gchar			*PathShareProjet = NULL;\n' )
					fp.write( '	gchar			*Path = NULL;\n' )
					fp.write( '	GtkBuilder		*GtkBuilderLocal = NULL;\n' )
					fp.write( '	// if exist alloc return ok\n' )
					fp.write( '	if( NULL != p_GtkBuilder ) {\n' )
					fp.write( '		return( p_GtkBuilder );\n' )
					fp.write( '	}\n' )
					fp.write( '	// if not init return\n' )
					fp.write( '	if( NULL == ( GtkBuilderLocal = gtk_builder_new() )) {\n' )
					fp.write( '		return( NULL );\n' )
					fp.write( '	}\n' )
					fp.write( '	gtk_builder_set_translation_domain( GtkBuilderLocal, GETTEXT_PACKAGE );\n' )
					fp.write( '	PathCurrentDir = g_strdup_printf( "%s/glade/%s", CurrentDir, p_name_glade );\n' )
					fp.write( '	g_free( CurrentDir );\n' )
					fp.write( '	CurrentDir = NULL;\n' )
					fp.write( '#ifdef ENABLE_XDG\n' )
					fp.write( '	PathShareProjet = xdg_search_data_xdg( p_name_glade );\n' )
					fp.write( '#else\n' )
					fp.write( '	PathShareProjet = g_strdup_printf( "%s/%s/%s", DATA_DIR, PACKAGE_NAME, p_name_glade );\n' )
					fp.write( '#endif\n' )
					fp.write( '	if( FALSE == g_file_test( PathCurrentDir, G_FILE_TEST_EXISTS ) && FALSE == g_file_test( PathShareProjet, G_FILE_TEST_EXISTS )) {\n' )
					fp.write( '		g_print( "!---------------------------------------------------------------------------------------------------\\n" );\n' )
					fp.write( '		g_print( "! LE FICHIER XML: [ %s ] N\'EXISTE PAS !!!\\n", PathCurrentDir );\n' )
					fp.write( '		g_print( "!---------------------------------------------------------------------------------------------------\\n" );\n' )
					fp.write( '		g_print( "! LE FICHIER XML: [ %s ] N\'EXISTE PAS !!!\\n", PathShareProjet );\n' )
					fp.write( '		g_print( "!---------------------------------------------------------------------------------------------------\\n" );\n' )
					fp.write( '		g_free( PathCurrentDir );\n' )
					fp.write( '		PathCurrentDir = NULL;\n' )
					fp.write( '		g_free( PathShareProjet );\n' )
					fp.write( '		PathShareProjet = NULL;\n' )
					fp.write( '		return( NULL );\n' )
					fp.write( '	}\n' )
					fp.write( '	else if( TRUE == g_file_test( PathCurrentDir, G_FILE_TEST_EXISTS )) {\n' )
					fp.write( '		Path = PathCurrentDir;\n' )
					fp.write( '	}\n' )
					fp.write( '	else if( TRUE == g_file_test( PathShareProjet, G_FILE_TEST_EXISTS )) {\n' )
					fp.write( '		Path = PathShareProjet;\n' )
					fp.write( '	}\n' )
					fp.write( '	g_print( "\\nPATHNAME FILE %s: %s\\n\\n", p_name_glade, Path );\n' )
					fp.write( '	if( gtk_builder_add_from_file( GtkBuilderLocal, Path, &p_err) <= 0 ) {\n' )
					fp.write( '		g_warning( "Couldn\'t load builder file: %s", p_err->message );\n' )
					fp.write( '		g_error_free( p_err );\n' )
					fp.write( '		g_free( PathCurrentDir );\n' )
					fp.write( '		PathCurrentDir = NULL;\n' )
					fp.write( '		g_free( PathShareProjet );\n' )
					fp.write( '		PathShareProjet = NULL;\n' )
					fp.write( '		Path = NULL;\n' )
					fp.write( '		return( NULL );\n' )
					fp.write( '	}\n' )
					fp.write( '	g_free( PathCurrentDir );\n' )
					fp.write( '	PathCurrentDir = NULL;\n' )
					fp.write( '	g_free( PathShareProjet );\n' )
					fp.write( '	PathShareProjet = NULL;\n' )
					fp.write( '	Path = NULL;\n' )
					fp.write( '	return( GtkBuilderLocal );\n' )
					fp.write( '}\n' )
					fp.write( '\n' )
					fp.close()
				#
				# create: <glade>.c
				# copy glade file to dest
				#
				for glade in self.ListNameFilesGlade:
					BaseName = os.path.basename( glade )
					Name = BaseName.split(".")[0]
					Ext  = BaseName.split(".")[-1]

					# append to po/POTFILES.in
					self.list_source_file.append( 'glade/%s' % BaseName )

					# copy glade file to dest
					shutil.copy2( glade, self.pathdest + "/glade/" + BaseName )
					#
					# create: <glade>.c
					#
					# append to po/POTFILES.in
					self.list_source_file.append( 'src/%s.c' % Name )
					fp = open( self.pathdest + "/" + src[0] + "/%s.c" % Name, "w" )
# 					fp.write( '/*\n' )
# 					fp.write( ' *  %s.c\n'  % Name )
# 					fp.write( ' */\n' )
# 					fp.write( '\n' )
					self.write_entete( fp, p_name_func='%s/%s.c' % (src[0], Name) )
					fp.write( '#ifdef HAVE_CONFIG_H\n' )
					fp.write( '	#include "../config.h"\n' )
					fp.write( '#endif\n' )
					fp.write( '#include <gtk/gtk.h>\n' )
					fp.write( '#include <glib.h>\n' )
					fp.write( '#include <string.h>\n' )
					fp.write( '#include <stdio.h>\n' )
					fp.write( '#include <stdlib.h>\n' )
					fp.write( '\n' )
					if self.name_lib_statique != "":
						fp.write( '#ifdef ENABLE_STATIC_LIBRARY\n' )
						fp.write( '	#include "../%s/lib.h"\n' % self.name_lib_statique )
						fp.write( '#endif\n' )
						fp.write( '\n' )
					fp.write( '#include "global.h"\n' )
					fp.write( '\n' )
					fp.write( '\n' )
					#
					codegen = codegen_py()
					codegen.create_callbacks( fp, glade, self.project_name )
					#
					fp.write( '\n' )
					fp.write( 'gboolean %s_init( GtkWidget *FromWindMain )\n' % Name )
					fp.write( '{\n' )
					fp.write( '\tgchar\t*PackageVersion = NULL;\n' )
					#
					codegen.create_code( fp, glade, self.project_name )
					#
					fp.write( '\treturn( TRUE );\n' )
					fp.write( '}\n' )
					fp.write( '\n' )
					fp.close()
			else:
				#
				# create: main.c --> NORMAL autre dossier que SRC
				#
				# append to po/POTFILES.in
				self.list_source_file.append( '%s/main.c' % src[0] )
				fp = open( self.pathdest + "/" + src[0] + "/main.c", "w" )
# 				fp.write( '/*\n' )
# 				fp.write( ' *  main.c\n' )
# 				fp.write( ' */\n' )
# 				fp.write( '\n' )
# 				fp.write( '\n' )
				self.write_entete( fp, p_name_func='%s/main.c' % src[0] )
				fp.write( '#ifdef HAVE_CONFIG_H\n' )
				fp.write( '	#include \"../config.h\"\n' )
				fp.write( '#endif\n' )
				fp.write( '\n' )
				fp.write( '#ifdef ENABLE_NLS\n' )
				fp.write( '	#include <libintl.h>\n' )
				fp.write( '	#define _(String) gettext (String)\n' )
				fp.write( '#endif\n' )
				fp.write( '\n' )
				if self.bool_gtk3:
					fp.write( '#include <gtk/gtk.h>\n' )
				if self.bool_glib:
					fp.write( '#include <glib.h>\n' )
					fp.write( '#ifdef ENABLE_STATIC_LIBRARY\n' )
					fp.write( '	#include "../%s/lib.h"\n' % self.name_lib_statique )
					fp.write( '#endif\n' )
				fp.write( '\n' )
				fp.write( '#include <stdlib.h>\n' )
				fp.write( '#include <stdio.h>\n' )
				fp.write( '\n' )
				if self.bool_glib:
					fp.write( '/* gint main (gint argc, gchar *argv[], gchar **envp)\n' )
				else:
					fp.write( '/* int main (int argc, char *argv[], char **envp)\n' )
				fp.write( ' *\n' )
				fp.write( ' * argc  designe le nombre d''arguments transmis au moment du lancement de l''executable\n' )
				fp.write( ' * argv  designe le vecteur contenant les differents arguments\n' )
				fp.write( ' * envp  designe le vecteur contenant les informations sur l''environnement\n' )
				fp.write( ' */\n' )
				if self.bool_glib:
					fp.write( 'gint main( gint argc, gchar *argv[], gchar **envp )\n' )
				else:
					fp.write( 'int main( int argc, char *argv[], char **envp )\n' )
				fp.write( '{\n' )
				fp.write( '\n' )
				fp.write( '\tprintf( "%s :: %s (line = %d)\\n", __FILE__, __FUNCTION__, __LINE__ );\n' )
				fp.write( '\tprintf( "\\n" );\n' )
				fp.write( '\tprintf( "***************************\\n" );\n' )
				fp.write( '\tprintf( "%s %s\\n", PACKAGE, VERSION );\n' )
				fp.write( '\tprintf( "***************************\\n" );\n' )
				fp.write( '\tprintf( "%s %s compiled at %s %s\\n", PACKAGE, VERSION, __DATE__, __TIME__ );\n' )
				fp.write( '\tprintf( "Compiler: gcc %s\\n", __VERSION__ );\n' )
				fp.write( '\tprintf( "***************************\\n" );\n' )
				fp.write( '#ifdef ENABLE_NLS\n' )
				fp.write( '\tprintf( "ENABLE_NLS OK\\n" );\n' )
				fp.write( '\tprintf( "\tGETTEXT_PACKAGE    = %s\\n", GETTEXT_PACKAGE );\n' )
				fp.write( '\tprintf( "\tLOCALE_DIR         = %s\\n", LOCALE_DIR );\n' )
				fp.write( '\tsetlocale( LC_ALL, "" );\n' )
				fp.write( '\tbindtextdomain( GETTEXT_PACKAGE, LOCALE_DIR );\n' )
				fp.write( '\tbind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );\n' )
				fp.write( '\ttextdomain( GETTEXT_PACKAGE );\n' )
				fp.write( '#endif\n' )
				fp.write( '\n' )
				if self.ListUpdateLangs:
					fp.write( '\tprintf( _("Hello world !\\n") );\n' )
				else:
					fp.write( '\tprintf( "Hello world !\\n" );\n' )
				fp.write( '\n' )
				fp.write( '\texit( EXIT_SUCCESS );\n' )
				fp.write( '}\n\n' )
				fp.close()
			#
			# create: src/Makefile.am
			#
			fp = open( self.pathdest + "/" + src[0] + "/Makefile.am", "w" )
			fp.write( '#\n# %s/Makefile.am with BaseBuild\n#\n' % src[0] )
			fp.write( 'AUTOMAKE_OPTIONS=gnu\n' )
			if self.bool_gtk3 or self.bool_glib:
				fp.write( '\nAM_CPPFLAGS = \\\n' )
				fp.write( '   -DPACKAGE_DATA_DIR=\\\"\"$(datadir)\"\\\" \\\n' )
				fp.write( '   -DPACKAGE_LOCALE_DIR=\\\"\"$(prefix)/$(DATADIRNAME)/locale\"\\\" \\\n' )
				fp.write( '   -DPIXMAPS_DIR=\\\"\"$(datadir)/pixmaps/%s\"\\\" \\\n' % self.project_name )
				fp.write( '	@PACKAGE_CFLAGS@\n' )
				fp.write( '\nAM_CFLAGS = @PACKAGE_CFLAGS@ \\\n' )
				fp.write( '   -DDATA_DIR=\\\"\"$(datadir)\"\\\" \\\n' )
				fp.write( '   -DLOCALE_DIR=\\\"\"$(datadir)/locale\"\\\" \\\n' )
				fp.write( '   -DPIXMAPS_DIR=\\\"\"$(datadir)/pixmaps/%s\"\\\" \n' % self.project_name )
			fp.write( '\nbin_PROGRAMS=%s\n' % src[1] )
			fp.write( '%s_SOURCES=main.c ' % src[1].replace("-", "_", -1) )
			if src[0] == "src":
				for glade in self.ListNameFilesGlade:
					BaseName = os.path.basename( glade )
					Name = BaseName.split(".")[0]
					Ext  = BaseName.split(".")[-1]
					fp.write( '%s.c ' % Name )
				if self.ListNameFilesGlade:
					fp.write( 'builder.c ' )
				fp.write( 'global.h ' )
			fp.write( '\n' )
			if self.name_lib_statique != "":
				fp.write( '\n%s_CPPFLAGS=-I${top_srcdir}/%s\n' % (src[1].replace("-", "_", -1), self.name_lib_statique) )
			if self.ListUpdateLangs:
				fp.write( '\ndatadir = @datadir@\n' )
				fp.write( 'localedir = $(datadir)/locale\n' )
				fp.write( 'DEFS = -DLOCALEDIR=\"$(localedir)\" @DEFS@\n' )
			if self.name_lib_statique != "" or self.bool_gtk3 or self.bool_glib or self.ListUpdateLangs:
				fp.write( '\n%s_LDADD=' % src[1].replace("-", "_", -1) )
				if self.name_lib_statique != "":
					fp.write( '${top_builddir}/%s/%s.a -lm' % (self.name_lib_statique, self.name_lib_statique) )
				if self.bool_gtk3 or self.bool_glib:
					fp.write( ' @PACKAGE_LIBS@' )
				if self.ListUpdateLangs:
					fp.write( ' $(INTLLIBS)' )
				fp.write( '\n' )
			fp.close()

	def write_folder_manpage(self):
		print( 'write_folder_manpage' )
		if self.bool_manpage:
			os.makedirs( self.pathdest + '/man' )
			#
			# Makefile.am
			#
			fp = open( self.pathdest + "/man/Makefile.am", "w" )
			fp.write( '#\n# man/Makefile.am with BaseBuild\n#\n' )
			fp.write( 'AUTOMAKE_OPTIONS=gnu\n' )
			fp.write( 'man1_MANS = %s.1.gz\n' % self.project_name )
			fp.write( 'EXTRA_DIST = compilate_manpage.txt %s.1.gz\n' % self.project_name )
			fp.close()
			#
			# man file
			#
			fp = open( self.pathdest + "/man/%s.1" % self.project_name, "w" )
			fp.write( '.\\" %s\n' % self.project_name )
			fp.write( '.\\" Copyright (C) %s %s\n' % (date.today().year, self.project_author) )
			fp.write( '.\\"\n' )
			fp.write( '.\\" %s! is free software; you can redistribute it and/or modify\n' % self.project_name )
			fp.write( '.\\" it under the terms of the GNU General Public License as published by\n' )
			fp.write( '.\\" the Free Software Foundation; either version 2, or\n' )
			fp.write( '.\\" (at your option) any later version.\n' )
			fp.write( '.\\"\n' )
			fp.write( '.\\" %s! is distributed in the hope that it will be useful,\n'  % self.project_name )
			fp.write( '.\\" but WITHOUT ANY WARRANTY; without even the implied warranty of\n' )
			fp.write( '.\\" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU\n' )
			fp.write( '.\\" General Public License for more details.\n' )
			fp.write( '.\\"\n' )
			fp.write( '.\\" You should have received a copy of the GNU General Public License\n' )
			fp.write( '.\\" along with program; see the file COPYING. If not, write to the\n' )
			fp.write( '.\\" Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,\n' )
			fp.write( '.\\" MA 02111-1307, USA.\n' )
			fp.write( '.\\"\n' )
			fp.write( '\n' )
			fp.write( '.TH %s 1 "Thu, 06 Mar 2014" %s "Manual: %s"\n' % (self.project_name, self.project_version, self.project_version))
			fp.write( '\n' )
			fp.write( '.SH NAME\n' )
			fp.write( '\\fB%s\\fP \- Short description of the program\n' % self.project_name )
			fp.write( '.br\n' )
			fp.write( '\n' )
			fp.write( '.SH AUTHOR\n' )
			fp.write( '\\fB%s\\fP is developed by %s\n' % (self.project_name, self.project_author) )
			fp.close()
			#
			# compilate_manpage.txt
			#
			fp = open( self.pathdest + "/man/compilate_manpage.txt", "w" )
			fp.write( '\n' )
			fp.write( 'nroff -man ./%s.1 | less\n' % self.project_name )
			fp.write( 'gzip --best ./%s.1\n' % self.project_name )
			fp.write( '\n' )
			fp.close()
			#
			# gzip --best ./basebuild.1
			#
			if os.path.isfile( '/usr/bin/gzip' ) == True:
				os.chdir( self.pathdest + "/man" )
				commande = "gzip --best %s.1" % self.project_name
				subprocess.call( commande, shell=True )

	def write_folder_pixmaps(self):
		print( 'write_folder_pixmaps' )
		if self.bool_pixmaps:
			os.makedirs( self.pathdest + '/pixmaps' )
			#
			# Makefile.am
			#
			fp = open( self.pathdest + "/pixmaps/Makefile.am", "w" )
			fp.write( '#\n# pixmaps/Makefile.am with BaseBuild\n#\n' )
			fp.write( 'AUTOMAKE_OPTIONS=gnu\n' )
			fp.write( 'pixmaps_DATA = %s.png\n' % self.project_name )
			fp.write( 'pixmapsdir = $(datadir)/pixmaps\n' )
			fp.write( 'EXTRA_DIST = $(pixmaps_DATA)\n' )
			fp.close()
			#
			# copy pixmap.png
			#
			if os.path.isfile( FConfig.DATADIR + "/basebuildc.png" ) == True:
				shutil.copy2( FConfig.DATADIR + "/basebuildc.png", self.pathdest + '/pixmaps/%s.png' % self.project_name )

	def write_folder_menu(self):
		print( 'write_folder_menu' )
		if self.name_menu != '':
			os.makedirs( self.pathdest + '/menu' )
			#
			# Makefile.am
			#
			fp = open( self.pathdest + "/menu/Makefile.am", "w" )
			fp.write( '#\n# menu/Makefile.am with BaseBuild\n#\n' )
			fp.write( 'AUTOMAKE_OPTIONS=gnu\n' )
			fp.write( 'menu_DATA = %s.desktop\n' % self.project_name )
			fp.write( 'menudir = $(datadir)/applications\n' )
			fp.write( 'EXTRA_DIST = $(menu_DATA)\n' )
			fp.close()
			#
			# file.desktop
			#
			fp = open( self.pathdest + "/menu/%s.desktop" % self.project_name, "w" )
			fp.write( '#!/usr/bin/env xdg-open\n' )
			Description = _("Short description of the program")
			fp.write( '[Desktop Entry]\n' )
			fp.write( 'Name=%s\n' % self.project_name )
			fp.write( 'Name[fr]=%s\n' % self.project_name )
			fp.write( 'Name[en]=%s\n' % self.project_name )
			fp.write( 'Name[en_GB]=%s\n' % self.project_name )
			fp.write( 'Comment=%s\n' % Description )
			fp.write( 'Comment[fr]=%s\n' % Description )
			fp.write( 'Comment[en]=%s\n' % Description )
			fp.write( 'Comment[en_GB]=%s\n' % Description )
			fp.write( 'GenericName=%s\n' % self.project_name )
			fp.write( 'GenericName[fr]=%s\n' % self.project_name )
			fp.write( 'GenericName[en]=%s\n' % self.project_name )
			fp.write( 'GenericName[en_GB]=%s\n' % self.project_name )
			fp.write( 'Icon=%s.png\n' % self.project_name)
			fp.write( 'Type=Application\n' )
			fp.write( 'Categories=Application;%s\n' % self.name_menu )
			fp.write( 'Encoding=UTF-8\n' )
			fp.write( 'Exec=%s\n' % self.project_name )
			fp.write( 'Terminal=false\n' )
			fp.write( 'MultipleArgs=false\n' )
			fp.write( 'StartupNotify=false\n' )
			fp.write( 'X-KDE-SubstituteUID=false\n' )
			fp.write( 'X-KDE-Username=\n' )
			fp.close()

	def write_folder_mime(self):
		print( 'write_folder_mime' )
		if self.name_mime != '':
			os.makedirs( self.pathdest + '/mime' )
			#
			# Makefile.am
			#
			fp = open( self.pathdest + "/mime/Makefile.am", "w" )
			fp.write( '#\n# mime/Makefile.am with BaseBuild\n#\n' )
			fp.write( 'AUTOMAKE_OPTIONS=gnu\n' )
			fp.write( 'EXTRA_DIST = %s\n' % self.project_name )
			fp.write( 'mime_DATA = %s\n' % self.project_name )
			fp.write( 'mimedir = $(prefix)/lib/mime/packages\n' )
			fp.close()
			#
			# mime_file
			#
			fp = open( self.pathdest + "/mime/%s" % self.project_name, "w" )
			# Text
			if self.name_mime == 'Text':
				fp.write( '# For text\n' )
				fp.write( 'text/plain; %s ''%%s'' ; edit=%s ''%%s'' ; test=test \"$DISPLAY\" != \"\" ; description=\"Text file\" ; priority=2\n' % (self.project_name, self.project_name) )
			# Audio
			if self.name_mime == 'Audio':
				fp.write( '# For Audio\n' )
				fp.write( 'application/ogg; %s ''%%s''; edit=%s ''%%s''; test=test \"$DISPLAY\" != \"\"; priority=2\n' % (self.project_name, self.project_name) )
				fp.write( 'audio/basic; %s ''%%s''; edit=%s ''%%s''; test=test \"$DISPLAY\" != \"\"; priority=2\n' % (self.project_name, self.project_name) )
				fp.write( 'audio/mpeg; %s ''%%s''; edit=%s ''%%s''; test=test \"$DISPLAY\" != \"\"; priority=2\n' % (self.project_name, self.project_name) )
				fp.write( 'audio/x-aiff; %s ''%%s''; edit=%s ''%%s''; test=test \"$DISPLAY\" != \"\"; priority=2\n' % (self.project_name, self.project_name) )
				fp.write( 'audio/x-mp3; %s ''%%s''; edit=%s ''%%s''; test=test \"$DISPLAY\" != \"\"; priority=2\n' % (self.project_name, self.project_name) )
				fp.write( 'audio/x-wav; %s ''%%s''; edit=%s ''%%s''; test=test \"$DISPLAY\" != \"\"; priority=2\n' % (self.project_name, self.project_name) )
				fp.write( 'application/x-flac; %s ''%%s''; edit=%s ''%%s''; test=test \"$DISPLAY\" != \"\"; priority=2\n' % (self.project_name, self.project_name) )
				fp.write( 'audio/x-flac; %s ''%%s''; edit=%s ''%%s''; test=test \"$DISPLAY\" != \"\"; priority=2\n' % (self.project_name, self.project_name) )
			# Video
			if self.name_mime == 'Video':
				fp.write( '# For Video\n' )
				fp.write( 'video/mpeg; %s ''%%s''; description=\"MPEG Video Format\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/quicktime; %s ''%%s''; description=\"Apple Quicktime Video\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-mpeg2; %s ''%%s''; description=\"MPEG Video\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-mpeg; %s ''%%s''; description=\"MPEG 2 Video\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/mpeg; %s ''%%s''; description=\"MPEG 2 Video\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-ms-afs; %s ''%%s''; description=\"Audio Video Interleave - Microsoft\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-ms-asf; %s ''%%s''; description=\"MS ASF video\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-msvideo; %s ''%%s''; description=\"Audio Video Interleave - Microsoft\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-ms-wma; %s ''%%s''; description=\"Audio Video Interleave - Microsoft\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-ms-wmv; %s ''%%s''; description=\"Audio Video Interleave - Microsoft\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-ms-wmx; %s ''%%s''; description=\"Audio Video Interleave - Microsoft\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
				fp.write( 'video/x-ms-wvx; %s ''%%s''; description=\"Audio Video Interleave - Microsoft\"; test=test -n \"$DISPLAY\"\n' % self.project_name )
			fp.close()


	def write_configure_ac(self):
		print( 'write_configure_ac' )
		#
		#
		#
		fp = open( self.pathdest + "/configure.ac", "w" )
		fp.write( '#\n# configure.in with BaseBuild\n#\n' )
		fp.write( '# -*- Autoconf -*-\n' )
		fp.write( '# Process this file with autoconf to produce a configure script.\n' )
		fp.write( '\n' )

		fp.write( 'AC_INIT([%s],\n\t[%s],\n\t[http://www.xcfa.tuxfamily.org/],\n\t[%s],\n\t[http://www.xcfa.tuxfamily.org/])\n' % (self.project_name, self.project_version, self.project_name) )
		fp.write( 'AM_INIT_AUTOMAKE\n' )
		fp.write( 'AC_CONFIG_SRCDIR([src/main.c])\n' )
		fp.write( 'AC_CONFIG_HEADERS([config.h])\n' )
		fp.write( '\n' )
		fp.write( '# Checks for programs.\n' )
		fp.write( 'AC_PROG_CXX\n' )
		fp.write( 'AC_PROG_CC\n' )
		fp.write( 'AC_PROG_CPP\n' )
		fp.write( 'AC_PROG_INSTALL\n' )
		fp.write( '# AC_PROG_LN_S\n' )
		fp.write( 'AC_PROG_MAKE_SET\n' )
		fp.write( '\n' )
		if self.name_lib_statique != "":
			fp.write( 'AC_DEFINE(ENABLE_XDG,1,[Define for enable xdg])\n' )
			fp.write( '# AM_PROG_LIBTOOL\n' )
			fp.write( 'AC_PROG_RANLIB\n' )
			fp.write( 'AM_PROG_CC_C_O\n' )
			fp.write( 'AC_DEFINE(ENABLE_STATIC_LIBRARY,1,[Define for enable static library])\n' )
		fp.write( '\n# Enable silent rules if supported\n' )
		fp.write( 'm4_ifdef([AM_SILENT_RULES], [AM_SILENT_RULES([yes])])\n' )
		fp.write( '\n# Set PREFIX: ./configure  OR ./configure --prefix=/usr\n' )
		fp.write( 'AC_SUBST(INSTALL_PREFIX)\n' )
		fp.write( 'AC_PREFIX_DEFAULT(/usr/local)\n' )
		fp.write( 'if [[ $prefix = \"NONE\" ]] ; then\n' )
		fp.write( '\tAC_DEFINE(INSTALL_PREFIX, \"/usr/local\",\n' )
		fp.write( '\t\t[define as prefix where %s is installed])\n' % self.project_name )
		fp.write( 'else\n' )
		fp.write( '\tAC_DEFINE_UNQUOTED(INSTALL_PREFIX, \"$prefix\")\n' )
		fp.write( 'fi\n' )
		if self.ListNameFilesGlade:
			fp.write( '\n' )
			fp.write( '# Set used name package glade: *.glade\n' )
			fp.write( 'AC_DEFINE(PACKAGE_NAME_GLADE, \"%s.glade\", [define as package name of glade])\n' % self.project_name )
		if self.ListUpdateLangs:
			fp.write( '\n# nls support\n' )
			fp.write( 'AC_MSG_CHECKING(for nls mode support)\n' )
			fp.write( 'BASEBUILD_LANG=\"en fr de it\"\n' )
			fp.write( 'AC_ARG_ENABLE(nls, , [\n' )
			fp.write( 'if test \"$enableval\" = \"yes\"; then\n' )
			fp.write( '\tALL_LINGUAS=$BASEBUILD_LANG\n' )
			fp.write( '\tAC_MSG_RESULT(yes)\n' )
			fp.write( '\tAM_GNU_GETTEXT([external])\n' )
			fp.write( '\tAC_DEFINE(ENABLE_NLS,1,[Define for NLS support])\n' )
			fp.write( 'else\n' )
			fp.write( '\tAC_MSG_RESULT(no)\n' )
			fp.write( '\tALL_LINGUAS=""\n' )
			fp.write( '\tUSE_NLS=\"no\"\n' )
			fp.write( 'fi\n' )
			fp.write( '], [\n' )
			fp.write( '\tALL_LINGUAS=$BASEBUILD_LANG\n' )
			fp.write( '\tAC_MSG_RESULT(yes)\n' )
			fp.write( '\tAM_GNU_GETTEXT([external])\n' )
			fp.write( '\tAC_DEFINE(ENABLE_NLS,1,[Define for NLS support])\n' )
			fp.write( '])\n\n' )
			fp.write( 'GETTEXT_PACKAGE=[$PACKAGE]\n' )
			fp.write( 'AC_SUBST(GETTEXT_PACKAGE)\n' )
			fp.write( 'AC_DEFINE_UNQUOTED([GETTEXT_PACKAGE],[\"$GETTEXT_PACKAGE\"],[gettext domain])\n' )
		fp.write( '\n# Checks for libraries.\n' )
		if self.name_lib_statique != "":
			#fp.write( 'AC_CHECK_LIB([%s], [libH_print_mess])\n' % self.name_lib_statique )
			fp.write( 'AC_CHECK_LIB([lib], [libH_print_mess])\n' )
		fp.write( '\n# Checks for header files.\n' )
		fp.write( 'AC_HEADER_STDC\n' )
		fp.write( 'AC_CHECK_HEADERS([libintl.h stdlib.h string.h unistd.h])\n' )
		fp.write( '\n# Checks for typedefs, structures, and compiler characteristics.\n' )
		fp.write( '\n# Checks for library functions.\n' )
		fp.write( 'AC_FUNC_STAT\n' )
		fp.write( 'AC_CHECK_FUNCS([setlocale strchr strrchr])\n' )


		GstrSrc = ""
		GstrSrc += '\n#\n'
		GstrSrc += '# debug and ansi support\n'
		GstrSrc += '#\n'
		GstrSrc += 'AC_ARG_ENABLE(debug_pg, AS_HELP_STRING([--disable-debug-pg], [Enable debug mode (default=disable)]),,enable_debug_pg=no)\n'
		GstrSrc += 'AC_ARG_ENABLE(debug, AS_HELP_STRING([--disable-debug], [Disable debug mode (default=disable)]),,enable_debug=no)\n'
		GstrSrc += 'AC_ARG_ENABLE(ansi,  AS_HELP_STRING([--disable-ansi],  [Enable ansi (default=enabled)]),,enable_ansi=no)\n'
		GstrSrc += '\n'
		GstrSrc += 'AC_MSG_CHECKING(for debug mode request)\n'
		GstrSrc += 'if test \"$enable_debug\" = \"yes\"; then\n'
		GstrSrc += ' 	AC_MSG_RESULT(yes)\n'
		GstrSrc += '	AC_DEFINE(DEBUG,1,[Define for debug support])\n'
		GstrSrc += '	AC_MSG_CHECKING(With ansi compilate)\n'
		GstrSrc += '	if test \"$enable_ansi\" = \"yes\"; then\n'
		GstrSrc += '		AC_MSG_RESULT(yes)\n'
		GstrSrc += '		AC_DEFINE(ENABLE_ANSI,1,[Define With ansi compilate])\n'
		GstrSrc += '		AC_MSG_CHECKING(With enable_debug_pg)\n'
		GstrSrc += '		if test \"$enable_debug_pg\" = \"yes\"; then\n'
		GstrSrc += '			AC_MSG_RESULT(yes)\n'
		GstrSrc += '			AC_DEFINE(ENABLE_DEBUG_PG,1,[Define for debug gprof support])\n'
		if self.bool_gtk3 or self.bool_glib:
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=yes, enable_ansi=yes, enable_debug_pg=yes\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-pg -g -O2 -Wall -ansi -D_REENTRANT -DLinux\"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-pg -g -O2 -Wall -ansi -D_REENTRANT -DLinux -Wl,--export-dynamic\"\n'
			GstrSrc += '		else\n'
			GstrSrc += '			AC_MSG_RESULT(no)\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=yes, enable_ansi=yes, enable_debug_pg=no\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-g -O2 -Wall -ansi -D_REENTRANT -DLinux\"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-g -O2 -Wall -ansi -D_REENTRANT -DLinux -Wl,--export-dynamic\"\n'
		else:
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=yes, enable_ansi=yes, enable_debug_pg=yes\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-pg -g -O2 -Wall -ansi \"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-pg -g -O2 -Wall -ansi\"\n'
			GstrSrc += '		else\n'
			GstrSrc += '			AC_MSG_RESULT(no)\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=yes, enable_ansi=yes, enable_debug_pg=no\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-g -O2 -Wall -ansi \"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-g -O2 -Wall -ansi\"\n'
		GstrSrc += '		fi\n'
		GstrSrc += '	else\n'
		GstrSrc += '		AC_MSG_RESULT(no)\n'
		GstrSrc += '		AC_MSG_CHECKING(With enable_debug_pg)\n'
		GstrSrc += '		if test \"$enable_debug_pg\" = \"yes\"; then\n'
		GstrSrc += '			AC_MSG_RESULT(yes)\n'
		GstrSrc += '			AC_DEFINE(ENABLE_DEBUG_PG,1,[Define for debug gprof support])\n'
		if self.bool_gtk3 or self.bool_glib:
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=yes, enable_ansi=no, enable_debug_pg=yes\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-pg -g -O2 -Wall -D_REENTRANT -DLinux\"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-pg -g -O2 -Wall -D_REENTRANT -DLinux -Wl,--export-dynamic\"\n'
			GstrSrc += '		else\n'
			GstrSrc += '			AC_MSG_RESULT(no)\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=yes, enable_ansi=no, enable_debug_pg=no\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-g -O2 -Wall -D_REENTRANT -DLinux\"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-g -O2 -Wall -D_REENTRANT -DLinux -Wl,--export-dynamic\"\n'
		else:
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=yes, enable_ansi=no, enable_debug_pg=yes\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-pg -g -O2 -Wall \"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-pg -g -O2 -Wall  \"\n'
			GstrSrc += '		else\n'
			GstrSrc += '			AC_MSG_RESULT(no)\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=yes, enable_ansi=no, enable_debug_pg=no\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-g -O2 -Wall \"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-g -O2 -Wall  \"\n'
		GstrSrc += '		fi\n'
		GstrSrc += '	fi\n'
		GstrSrc += 'else\n'
		GstrSrc += ' 	AC_MSG_RESULT(no)\n'
		GstrSrc += '	AC_MSG_CHECKING(With ansi compilate)\n'
		GstrSrc += ' 	if test \"$enable_ansi\" = \"yes\"; then\n'
		GstrSrc += ' 		AC_MSG_RESULT(yes)\n'
		GstrSrc += '		AC_DEFINE(ENABLE_ANSI,1,[Define With ansi compilate])\n'
		GstrSrc += '		AC_MSG_CHECKING(With enable_debug_pg)\n'
		GstrSrc += '		if test \"$enable_debug_pg\" = \"yes\"; then\n'
		GstrSrc += '			AC_MSG_RESULT(yes)\n'
		GstrSrc += '			AC_DEFINE(ENABLE_DEBUG_PG,1,[Define for debug gprof support])\n'
		if self.bool_gtk3 or self.bool_glib:
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=no, enable_ansi=yes, enable_debug_pg=yes\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-pg -Wall -ansi -D_REENTRANT -DLinux\"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-pg -Wall -ansi -D_REENTRANT -DLinux -Wl,--export-dynamic\"\n'
			GstrSrc += '		else\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=no, enable_ansi=yes, enable_debug_pg=no\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-Wall -ansi -D_REENTRANT -DLinux\"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-Wall -ansi -D_REENTRANT -DLinux -Wl,--export-dynamic\"\n'
		else:
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=no, enable_ansi=yes, enable_debug_pg=yes\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-pg -Wall -ansi \"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-pg -Wall -ansi\"\n'
			GstrSrc += '		else\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=no, enable_ansi=yes, enable_debug_pg=no\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-Wall -ansi \"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-Wall -ansi\"\n'
		GstrSrc += '		fi\n'
		GstrSrc += '	else\n'
		GstrSrc += '		AC_MSG_RESULT(no)\n'
		GstrSrc += '		AC_MSG_CHECKING(With enable_debug_pg)\n'
		GstrSrc += '		if test \"$enable_debug_pg\" = \"yes\"; then\n'
		GstrSrc += '			AC_MSG_RESULT(yes)\n'
		GstrSrc += '			AC_DEFINE(ENABLE_DEBUG_PG,1,[Define for debug gprof support])\n'
		if self.bool_gtk3 or self.bool_glib:
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=no, enable_ansi=no, enable_debug_pg=yes\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-pg -Wall -D_REENTRANT -DLinux\"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-pg -Wall -D_REENTRANT -DLinux -Wl,--export-dynamic\"\n'
			GstrSrc += '		else\n'
			GstrSrc += '			AC_MSG_RESULT(no)\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=no, enable_ansi=no, enable_debug_pg=no\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-Wall -D_REENTRANT -DLinux\"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-Wall -O3 -D_REENTRANT -DLinux -Wl,--export-dynamic\"\n'
		else:
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=no, enable_ansi=no, enable_debug_pg=yes\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-pg -Wall \"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-pg -Wall  \"\n'
			GstrSrc += '		else\n'
			GstrSrc += '			AC_MSG_RESULT(no)\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			echo \"*** enable_debug=no, enable_ansi=no, enable_debug_pg=no\"\n'
			GstrSrc += '			echo \"*** CFLAGS=-Wall \"\n'
			GstrSrc += '			echo \"***\"\n'
			GstrSrc += '			CFLAGS=\"-Wall -O3  \"\n'
		GstrSrc += '		fi\n'
		GstrSrc += '	fi\n'
		GstrSrc += 'fi\n'
		GstrSrc += '\n'
		fp.write( GstrSrc )
		#
		# IF USE math.h MODULE:
		#
		fp.write( '#\n' )
		fp.write( '# Check for complex math functions support: -lm\n' )
		fp.write( '#\n' )
		fp.write( 'AC_CHECK_LIB(m,roundf,[\n' )
		fp.write( '  LDFLAGS=\"-lm $LDFLAGS\"\n' )
		fp.write( '])\n' )

		if self.bool_gtk3 or self.bool_glib:
			fp.write( '#\n' )
			fp.write( '# ' )
			if self.bool_gtk3:
				fp.write( 'gtk+-3.0 ' )
			if self.bool_glib:
				fp.write( 'glib-2.0' )
			fp.write( '\n#\n' )
			fp.write( 'pkg_modules="' )
			if self.bool_gtk3:
				fp.write( 'gtk+-3.0 ' )
			if self.bool_glib:
				fp.write( 'glib-2.0' )
			fp.write( '"\n' )
			fp.write( 'PKG_CHECK_MODULES(PACKAGE, [$pkg_modules])\n' )
			fp.write( 'AC_SUBST(PACKAGE_CFLAGS)\n' )
			fp.write( 'AC_SUBST(PACKAGE_LIBS)\n' )

		GstrSrc = '\nAC_CONFIG_FILES([\n'
		if self.ListUpdateLangs:
			GstrSrc += 'po/Makefile.in\n'
		if self.name_lib_statique:
			GstrSrc += '%s/Makefile\n' % self.name_lib_statique
		GstrSrc += 'Makefile\n'
		for src in self.ListScrFolders:
			GstrSrc += '%s/Makefile\n'  % src[0]
		if self.bool_manpage:
			GstrSrc += 'man/Makefile\n'
		if self.bool_pixmaps:
			GstrSrc += 'pixmaps/Makefile\n'
		if self.name_menu:
			GstrSrc += 'menu/Makefile\n'
		if self.name_mime:
			GstrSrc += 'mime/Makefile\n'
		if self.ListNameFilesGlade:
			GstrSrc += 'glade/Makefile\n'
		GstrSrc += '])\n'
		fp.write( GstrSrc )

		GstrSrc = '\nAC_OUTPUT\n'
		GstrSrc += '\necho "\n'
		GstrSrc += '=================================================================\n'
		GstrSrc += '  configure.in product with BaseBuild: \n'
		GstrSrc += '      http://www.xcfa.tuxfamily.org/\n'
		GstrSrc += '=================================================================\n'
		GstrSrc += '  Configuration for $PACKAGE $VERSION :\n'
		GstrSrc += '=================================================================\n\n'
		GstrSrc += '  Host System Type ................ : $host\n'
		GstrSrc += '  HosName ......................... : `hostname`\n'
		GstrSrc += '  Compiler ........................ : $CC $CPPFLAGS\n'
		GstrSrc += '  Linker .......................... : $CC $LDFLAGS $LIB'
		if self.bool_gtk3:
			GstrSrc += ' $LIBS'
		GstrSrc += '\n'
		if self.ListUpdateLangs:
			GstrSrc += '  NLS/gettext ..................... : $USE_NLS\n'
			GstrSrc += '  ALL_LINGUAS ..................... : $ALL_LINGUAS\n'
		if self.bool_gtk3:
			GstrSrc += '  GTK+ version .................... : `pkg-config --modversion gtk+-3.0`\n'
		if self.bool_glib:
			GstrSrc += '  GLIB version .................... : `pkg-config --modversion glib-2.0`\n'
		GstrSrc += '  Debug mode g .................... : $enable_debug\n'
		GstrSrc += '  Debug mode pg ................... : $enable_debug_pg\n'
		GstrSrc += '  WITH ANSI COMPILATE ............. : $enable_ansi\n'
		GstrSrc += '  CFLAGS .......................... : $CFLAGS\n'
		GstrSrc += '  INSTALL ......................... : $prefix/bin/%s\n\n' % self.project_name
		GstrSrc += '  Options parameters:\n'
		GstrSrc += '  -------------------\n'
		GstrSrc += '  by default:\n'
		GstrSrc += '  	--disable-debug                             No debug support\n'
		GstrSrc += '  	--disable-debug-pg                          No debug support with gprof\n'
		GstrSrc += '  	--disable-ansi                              No define ansi for probleme compilate\n'
		GstrSrc += '  Options is:\n'
		GstrSrc += '  	--enable-debug      #define DEBUG 1         Define for debug: -g\n'
		GstrSrc += '  	--enable-debug-pg   #define DEBUG_PG 1      Define for debug support with gprof: -pg\n'
		GstrSrc += '  	--enable-ansi       #define ENABLE_ANSI 1   Define With ansi compilate\n\n'
		GstrSrc += '  Now type \'make\' to build $PACKAGE $VERSION,\n'
		GstrSrc += '  and then type (under root) \'make install\' for installation.\n'
		GstrSrc += '"\n'
		fp.write( GstrSrc )
		fp.close()

	def create_folder_po(self):
		print( 'create_folder_po' )
		if self.ListUpdateLangs:
			os.makedirs( self.pathdest + '/po' )
			#
			# POUR OUTREPASSER GETTEXTIZE
			# http://culot.org/public/Docs/autotools.html
			# gettextize copie tous les fichiers standards Gettext dans le répertoire principal d'un package,
			# pour commencer sa traduction.
			# Prise en compte de l'internationalisation qui modifie 'configure.in' et 'Makefile.am'
			#    $ gettextize -c
			# Le fichier ./ABOUT-NLS sera cree
			if os.path.isfile( '/tmp/my-gettextize.sh' ) == True:
				os.unlink( '/tmp/my-gettextize.sh' )
			subprocess.call( "sed 's:read .*< /dev/tty::' `which gettextize` > /tmp/my-gettextize.sh", shell=True )
			st = os.stat( '/tmp/my-gettextize.sh' )
			os.chmod( '/tmp/my-gettextize.sh', st.st_mode | stat.S_IEXEC)
			os.chdir( self.pathdest )
			subprocess.call( "/tmp/my-gettextize.sh -c", shell=True )
			os.unlink( '/tmp/my-gettextize.sh' )
			#
			# RENOMMER 'po/Makevars.template' vers 'po/Makevars'
			os.chdir( self.pathdest + '/po' )
			os.renames( 'Makevars.template', 'Makevars' )
			#
			# ADAPTATION DE 'po/POTFILES.in'
			fp = open( self.pathdest + "/po/POTFILES.in", "w" )
			fp.write( '# List of source files which contain translatable strings.\n' )
			for list_po in self.list_source_file:
				fp.write( list_po + '\n' )
			fp.close()
			#
			# create po/po.file
			#
			# self.ListUpdateLangs =  [['checkbutton_english', 'english', 'en'],
			for lang in self.ListUpdateLangs:
				GstrSrc  = '# SOME DESCRIPTIVE TITLE.\n'
				GstrSrc += '# Copyright (C) %s Free Software Foundation, Inc.\n' % date.today().year
				GstrSrc += '# This file is distributed under the same license as the PACKAGE package.\n'
				GstrSrc += '# FIRST AUTHOR <EMAIL@ADDRESS>, %s.\n' % date.today().year
				GstrSrc += '#\n'
				GstrSrc += 'msgid \"\"\n'
				GstrSrc += 'msgstr \"\"\n'
				GstrSrc += '\"Project-Id-Version: %s %s\\n\"\n' % (self.project_name, self.project_version)
				GstrSrc += '\"Report-Msgid-Bugs-To: Emailr@Adress.Ext\\n\"\n'

				po_date = datetime.datetime.now()
				GstrSrc += '\"POT-Creation-Date: %s\\n"\n' % po_date
				GstrSrc += '\"PO-Revision-Date: %s\\n"\n' % po_date
				#GstrSrc += '\"POT-Creation-Date: 2010-05-06 16:59+0200\\n\"\n'
				#GstrSrc += '\"PO-Revision-Date: 2010-05-06 16:59+0200\\n\"\n'

				GstrSrc += '\"Last-Translator: MyName <Emailr@Adress.Ext>\\n\"\n'
				GstrSrc += '\"Language-Team: %s\\n\"\n' % lang[ 1 ]
				GstrSrc += '\"MIME-Version: 1.0\\n\"\n'
				GstrSrc += '\"Content-Type: text/plain; charset=UTF-8\\n\"\n'
				GstrSrc += '\"Content-Transfer-Encoding: 8bit\\n\"\n'

				fp = open( self.pathdest + "/po/%s.po" % lang[ 2 ], "w" )
				fp.write( GstrSrc )
				fp.close()

	def write_autogen_sh(self):
		print( 'write_autogen_sh' )
		#
		# create autogen.sh
		#
		GstrSrc  = '#!/bin/sh\n'
		GstrSrc += '#\n# autogen.sh with BaseBuild\n#\n'
		GstrSrc += '\n'
		GstrSrc += '(aclocal --version) < /dev/null > /dev/null 2>&1 || {\n'
		GstrSrc += '  echo\n'
		GstrSrc += '  echo \"** Error **: You must have aclocal installed.\"\n'
		GstrSrc += '  echo\n'
		GstrSrc += '  exit 1\n'
		GstrSrc += '}\n'
		GstrSrc += '\n'
		GstrSrc += '(autoheader --version) < /dev/null > /dev/null 2>&1 || {\n'
		GstrSrc += '  echo\n'
		GstrSrc += '  echo \"** Error **: You must have autoheader installed.\"\n'
		GstrSrc += '  echo\n'
		GstrSrc += '  exit 1\n'
		GstrSrc += '}\n'
		GstrSrc += '\n'
		GstrSrc += '(autoconf --version) < /dev/null > /dev/null 2>&1 || {\n'
		GstrSrc += '  echo\n'
		GstrSrc += '  echo \"** Error **: You must have autoconf installed.\"\n'
		GstrSrc += '  echo\n'
		GstrSrc += '  exit 1\n'
		GstrSrc += '}\n'
		GstrSrc += '\n'
		GstrSrc += '(automake --version) < /dev/null > /dev/null 2>&1 || {\n'
		GstrSrc += '  echo\n'
		GstrSrc += '  echo \"** Error **: You must have aclocal installed.\"\n'
		GstrSrc += '  echo\n'
		GstrSrc += '  exit 1\n'
		GstrSrc += '}\n'
		GstrSrc += '\n'
		GstrSrc += 'echo aclocal\n'
		GstrSrc += 'aclocal\n'
		GstrSrc += 'echo autoheader\n'
		GstrSrc += 'autoheader\n'
		GstrSrc += 'echo autoconf\n'
		GstrSrc += 'autoconf\n'
		GstrSrc += 'echo automake -a -c\n'
		GstrSrc += 'automake -a -c\n'
		GstrSrc += 'echo automake --add-missing\n'
		GstrSrc += 'automake --add-missing\n'
		GstrSrc += 'echo automake\n'
		GstrSrc += 'automake\n'
		GstrSrc += 'echo ./configure --prefix=/usr\n'
		GstrSrc += './configure --prefix=/usr\n'
		GstrSrc += 'echo \n'
		GstrSrc += 'exit 0\n'
		fp = open( self.pathdest + "/autogen.sh", "w" )
		fp.write( GstrSrc )
		fp.close()
		st = os.stat( self.pathdest + "/autogen.sh" )
		os.chmod( self.pathdest + "/autogen.sh", st.st_mode | stat.S_IEXEC)

		#
		# create TODO, NEWS, README
		#
		fp = open( self.pathdest + "/TODO", "w" )
		fp.close()
		fp = open( self.pathdest + "/NEWS", "w" )
		fp.close()
		fp = open( self.pathdest + "/README", "w" )
		fp.close()
		#
		# create AUTHORS
		#
		if os.path.isfile( "/usr/bin/figlet" ) == True:
			command = 'figlet -w 200 -f small "    %s"' % self.project_name + ' > %s/' % self.pathdest + '/AUTHORS'
			subprocess.call(command, shell=True)
		else:
			fp = open( self.pathdest + "/AUTHORS", "w" )
			fp.write( '!---\n' )
			fp.write( '! %s : AUTHORS\n' % self.project_name )
			fp.write( '!---\n' )
			fp.close()
		#
		# create ChangeLog
		#
		if os.path.isfile( "/usr/bin/figlet" ) == True:
			command = 'figlet -w 200 -f small "    %s"' % self.project_name + ' > %s/' % self.pathdest + '/ChangeLog'
			subprocess.call(command, shell=True)
		else:
			fp = open( self.pathdest + "/ChangeLog", "w" )
			fp.write( '!---\n' )
			fp.write( '! %s : ChangeLog\n' % self.project_name )
			fp.write( '!---\n' )
			fp.close()
		#
		# add to ChangeLog
		#
		fp = open( self.pathdest + "/ChangeLog", "a" )
		fp.write( '\t========================================\n' )
		fp.write( '\t |                                    |\n' )
		fp.write( '\t |             Changelog              |\n' )
		fp.write( '\t |                                    |\n' )
		fp.write( '\t========================================\n' )
		fp.write( '\n' )
		fp.write( '\t========================================\n' )
		fp.write( '\t |  %s                             |\n' % self.project_version )
		fp.write( '\t========================================\n' )
		fp.write( '\t- Add comments\n' )
		fp.write( '\n' )
		fp.close()









