#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

# file      : wind_main.py
# project   : basebuildc
# copyright : (C) 2014 by Claude BULIN
#
# This file is part of basebuildc project
#
# basebuildc is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# basebuildc is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA


import sys, os
try:
	from gi.repository import Gtk, GObject, Gdk
except:
	print("GTK Not Availible")
	sys.exit(1)
import subprocess, threading

from fconfig import FConfig
import Language_Init
from prefs import Preferences
from wind_about import wind_about
from wind_info import wind_info
from wind_languages import wind_languages, wind_languages_init_list
from build import build_write
from code_gen import codegen_py


# constant variables for notebook
(NOTEBOOK_DEF_PROJECT, NOTEBOOK_ADD_LANGUAGES, NOTEBOOK_DEF_FOLDERS_PROJECT, NOTEBOOK_DEF_SRC_FOLDERS_PROJECT, NOTEBOOK_GENERATE) = range(5)


class wind_main():

	def __init__( self, name_glade_file="wind_main.glade" ):
	# def __init__( self, name_glade_file="wind_main.glade", Form=None ):

		# Add language support
		_ = Language_Init.Translator(FConfig).lang.gettext
		self._ = _

		try:
			self.builder = Gtk.Builder()
			# Add language support from glade file
			self.builder.set_translation_domain("basebuildc")
			self.builder.add_from_file( FConfig.DATADIR + "/" + name_glade_file )
		except:
			print("except from:Gtk.Builder() ")
			sys.exit(1)

		self.ListNameFilesGlade = []
		self.model_ListStore = None
		self.selection_ListStore = None
		self.timeout_id = -1

		self.wind_main=self.builder.get_object("wind_main")
		# if Form != None:
		#	self.wind_main.set_transient_for( Form )
		#	self.wind_main.set_modal( True )

		dic={
			"on_wind_main_delete_event": self.on_wind_main_delete_event,
			"on_wind_main_destroy_event": self.on_wind_main_destroy_event,
			"on_wind_main_key_press_event": self.on_wind_main_key_press_event,
			"on_wind_main_key_release_event": self.on_wind_main_key_release_event,
			"on_imagemenuitem_quit_activate": self.on_imagemenuitem_quit_activate,
			"on_menuitem_help_activate": self.on_menuitem_help_activate,
			"on_notebook_main_switch_page": self.on_notebook_main_switch_page,
			"on_entry_project_name_changed": self.on_entry_project_name_changed,
			"on_entry_project_version_changed": self.on_entry_project_version_changed,
			"on_entry_project_author_changed": self.on_entry_project_author_changed,
			"on_button_project_destination_clicked": self.on_button_project_destination_clicked,
			"on_label_drag_and_drop_realize": self.on_label_drag_and_drop_realize,
			"on_button_add_glade_file_clicked": self.on_button_add_glade_file_clicked,
			"on_button_clear_glade_file_clicked": self.on_button_clear_glade_file_clicked,
			"on_button_add_languages_clicked": self.on_button_add_languages_clicked,
			"on_combobox_folder_menu_realize": self.on_combobox_folder_menu_realize,
			"on_combobox_folder_mime_realize": self.on_combobox_folder_mime_realize,
			"on_button_generate_clicked": self.on_button_generate_clicked,
			"on_button_previous_clicked": self.on_button_previous_clicked,
			"on_button_next_clicked": self.on_button_next_clicked,
			"on_viewport_src_folders_realize": self.on_viewport_src_folders_realize,
			"on_button_add_folder_src_clicked": self.on_button_add_folder_src_clicked,
			"on_button_remove_folder_src_clicked": self.on_button_remove_folder_src_clicked,
			"on_checkbutton_folder_clicked": self.on_checkbutton_folder_clicked,
		}
		self.builder.connect_signals(dic)

		self.label_title_wind_main=self.builder.get_object("label_title_wind_main")
		self.label3=self.builder.get_object("label3")
		self.label4=self.builder.get_object("label4")
		self.label5=self.builder.get_object("label5")
		self.label6=self.builder.get_object("label6")
		self.entry_project_name=self.builder.get_object("entry_project_name")
		self.entry_project_version=self.builder.get_object("entry_project_version")
		self.entry_project_author=self.builder.get_object("entry_project_author")
		self.button_project_destination=self.builder.get_object("button_project_destination")
		self.label_name_dest_project=self.builder.get_object("label_name_dest_project")
		self.label2=self.builder.get_object("label2")
		self.label14=self.builder.get_object("label14")
		self.label_drag_and_drop=self.builder.get_object("label_drag_and_drop")
		self.button_add_glade_file=self.builder.get_object("button_add_glade_file")
		self.button_clear_glade_file=self.builder.get_object("button_clear_glade_file")
		self.label1=self.builder.get_object("label1")
		self.label_project_configuration_1=self.builder.get_object("label_project_configuration_1")
		self.label_project_configuration_2=self.builder.get_object("label_project_configuration_2")
		self.label_name_dest_project1=self.builder.get_object("label_name_dest_project1")
		self.label19=self.builder.get_object("label19")
		self.label10=self.builder.get_object("label10")
		self.button_add_languages=self.builder.get_object("button_add_languages")
		self.label15=self.builder.get_object("label15")
		self.label_see_languages_1=self.builder.get_object("label_see_languages_1")
		self.label_see_languages_2=self.builder.get_object("label_see_languages_2")
		self.label7=self.builder.get_object("label7")
		self.label_project_configuration_3=self.builder.get_object("label_project_configuration_3")
		self.label_project_configuration_4=self.builder.get_object("label_project_configuration_4")
		self.label_name_dest_project2=self.builder.get_object("label_name_dest_project2")
		self.label32=self.builder.get_object("label32")
		self.checkbutton_folder_manpage=self.builder.get_object("checkbutton_folder_manpage")
		self.checkbutton_folder_pixmaps=self.builder.get_object("checkbutton_folder_pixmaps")
		self.checkbutton_folder_lib=self.builder.get_object("checkbutton_folder_lib")
		self.checkbutton_folder_menu=self.builder.get_object("checkbutton_folder_menu")
		self.checkbutton_folder_mime=self.builder.get_object("checkbutton_folder_mime")
		self.checkbutton_lib_gtk3=self.builder.get_object("checkbutton_lib_gtk3")
		self.checkbutton_lib_glib=self.builder.get_object("checkbutton_lib_glib")
		self.label35=self.builder.get_object("label35")
		self.label36=self.builder.get_object("label36")
		self.entry_folder_lib=self.builder.get_object("entry_folder_lib")
		self.combobox_folder_menu=self.builder.get_object("combobox_folder_menu")
		self.combobox_folder_mime=self.builder.get_object("combobox_folder_mime")
		self.label33=self.builder.get_object("label33")
		self.entry_add_library=self.builder.get_object("entry_add_library")
		self.label48=self.builder.get_object("label48")
		self.label8=self.builder.get_object("label8")
		self.label_project_configuration_5=self.builder.get_object("label_project_configuration_5")
		self.label_project_configuration_6=self.builder.get_object("label_project_configuration_6")
		self.label_name_dest_project3=self.builder.get_object("label_name_dest_project3")
		self.label45=self.builder.get_object("label45")
		self.label47=self.builder.get_object("label47")
		self.button_add_folder_src=self.builder.get_object("button_add_folder_src")
		self.button_remove_folder_src=self.builder.get_object("button_remove_folder_src")
		self.label46=self.builder.get_object("label46")
		self.label12=self.builder.get_object("label12")
		self.label_project_configuration_7=self.builder.get_object("label_project_configuration_7")
		self.label_project_configuration_8=self.builder.get_object("label_project_configuration_8")
		self.label_name_dest_project4=self.builder.get_object("label_name_dest_project4")
		self.label57=self.builder.get_object("label57")
		self.label_see_project_generate=self.builder.get_object("label_see_project_generate")
		self.button_generate=self.builder.get_object("button_generate")
		self.label9=self.builder.get_object("label9")
		self.label34=self.builder.get_object("label34")
		self.button_previous=self.builder.get_object("button_previous")
		self.label11=self.builder.get_object("label11")
		self.button_next=self.builder.get_object("button_next")
		self.label_progressbar=self.builder.get_object("label_progressbar")
		self.progressbar_main=self.builder.get_object("progressbar_main")
		self.notebook_main=self.builder.get_object("notebook_main")
		self.box_previous_next=self.builder.get_object("box_previous_next")
		self.box_progressbar=self.builder.get_object("box_progressbar")
		self.scrolledwindow_see_project_generate=self.builder.get_object("scrolledwindow_see_project_generate")

		# header
		title_string = "Base Build C"
		comment_string = _("Generating a skeleton C from one or more files *.Glade")
		string = "<span font_desc=\"sans bold 20 \">\n<b>%s</b>\n</span><b>%s</b>" % (title_string, comment_string)
		self.label_title_wind_main.set_markup( string )
		self.wind_main.set_title("%s - Version %s" % (FConfig.__PACKAGE__, FConfig.__VERSION__))

		# Preferences: init
		self.Prefs = Preferences()
		self.Prefs.Preferences_read_file()
		# Preferences: set window position
		win_width      = self.Prefs.Preferences_get_win_width()
		win_height     = self.Prefs.Preferences_get_win_height()
		main_win_pos_x = self.Prefs.Preferences_get_main_win_pos_x()
		main_win_pos_y = self.Prefs.Preferences_get_main_win_pos_y()
		self.wind_main.resize( win_width, win_height )
		self.wind_main.move( main_win_pos_x, main_win_pos_y )
		# Preferences:
		self.entry_project_name.set_text( self.Prefs.Preferences_get_ProjectName() )
		self.entry_project_version.set_text( self.Prefs.Preferences_get_ProjectVersion() )
		self.entry_project_author.set_text( self.Prefs.Preferences_get_ProjectAuthor() )
		self.button_project_destination.set_label( self.Prefs.Preferences_get_form_ProjectDestination() )

		# Default languages
		_ListUpdateLangs = self.Prefs.Preferences_get_LANGUAGES()
		WindLang = wind_languages_init_list()
		self.ListUpdateLangs = WindLang.init_list( _ListUpdateLangs )
		self.Update_languages( self.ListUpdateLangs )

		# init notebook in reverse
		list_page_notebook = [ 4, 3, 2, 1, 0 ]
		for page_notebook in list_page_notebook:
			self.notebook_main.set_current_page( page_notebook )

		# set checkbutton
		self.checkbutton_folder_manpage.set_active( self.Prefs.Preferences_get_form_checkbutton_folder_manpage() )
		self.checkbutton_folder_pixmaps.set_active( self.Prefs.Preferences_get_form_checkbutton_folder_pixmaps() )
		self.checkbutton_folder_lib.set_active( self.Prefs.Preferences_get_form_checkbutton_folder_lib() )
		self.checkbutton_folder_menu.set_active( self.Prefs.Preferences_get_form_checkbutton_folder_menu() )
		self.checkbutton_folder_mime.set_active( self.Prefs.Preferences_get_form_checkbutton_folder_mime() )
		self.checkbutton_lib_gtk3.set_active( self.Prefs.Preferences_get_form_checkbutton_lib_gtk3() )
		self.checkbutton_lib_glib.set_active( self.Prefs.Preferences_get_form_checkbutton_lib_glib() )
		self.combobox_folder_menu.set_sensitive( self.checkbutton_folder_menu.get_active() )
		self.combobox_folder_mime.set_sensitive( self.checkbutton_folder_mime.get_active() )
		self.entry_folder_lib.set_sensitive( self.checkbutton_folder_lib.get_active() )

		self.box_progressbar.hide()

		self.wind_main.set_icon_from_file( FConfig.DATADIR + "/basebuildc.png" )

		self.wind_main.show()


	#
	# End of: Bye
	#
	def main_save_preferences(self):
		x, y = self.wind_main.get_position()
		width, height = self.wind_main.get_size()
		self.Prefs.Preferences_save_op(
			self.entry_project_name.get_text(),
			self.entry_project_version.get_text(),
			self.entry_project_author.get_text(),
			self.button_project_destination.get_label(),
			self.ListUpdateLangs,
			self.checkbutton_folder_manpage.get_active(),
			self.checkbutton_folder_pixmaps.get_active(),
			self.checkbutton_folder_lib.get_active(),
			self.checkbutton_folder_menu.get_active(),
			self.checkbutton_folder_mime.get_active(),
			self.checkbutton_lib_gtk3.get_active(),
			self.checkbutton_lib_glib.get_active(),

			self.combobox_folder_menu.get_active(),
			self.combobox_folder_mime.get_active(),

			x, y, width, height)

	def on_wind_main_delete_event(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()

	def on_wind_main_destroy_event(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()

	def on_imagemenuitem_quit_activate(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()


	#
	# key press/release
	#
	def on_wind_main_key_press_event(self, widget, data=None):
		pass
	def on_wind_main_key_release_event(self, widget, data=None):
		pass


	#
	# About
	#
	def on_menuitem_help_activate(self, widget, data=None):
		print( "on_menuitem_help_activate" )
		wind_about(Form=self.wind_main)


	#
	# Updating buttons
	#
	def config_buttons_actions(self):

		txt_name_prj = self.entry_project_name.get_text().strip()
		txt_version_prj = self.entry_project_version.get_text().strip()
		txt_author_prj = self.entry_project_author.get_text().strip()
		txt_dest_prj =  self.button_project_destination.get_label()
		txt_drag = self.label_drag_and_drop.get_label().strip()
		# txt_lang = self.label_see_languages.get_label().strip()

		self.builder.get_object("label_project_configuration_2").set_label(txt_name_prj + "\n" + txt_version_prj + "\n" + txt_author_prj + "\n" + txt_dest_prj )
		self.builder.get_object("label_project_configuration_4").set_label(txt_name_prj + "\n" + txt_version_prj + "\n" + txt_author_prj + "\n" + txt_dest_prj )
		self.builder.get_object("label_project_configuration_6").set_label(txt_name_prj + "\n" + txt_version_prj + "\n" + txt_author_prj + "\n" + txt_dest_prj )
		self.builder.get_object("label_project_configuration_8").set_label(txt_name_prj + "\n" + txt_version_prj + "\n" + txt_author_prj + "\n" + txt_dest_prj )

		path_name_file = self.button_project_destination.get_label() + "/" + txt_name_prj + "-" + txt_version_prj
		self.builder.get_object("label_name_dest_project").set_markup( "<span color=\"green\"><b>%s</b></span>" % path_name_file )
		self.builder.get_object("label_name_dest_project1").set_markup( "<span color=\"green\"><b>%s</b></span>" % path_name_file )
		self.builder.get_object("label_name_dest_project2").set_markup( "<span color=\"green\"><b>%s</b></span>" % path_name_file )
		self.builder.get_object("label_name_dest_project3").set_markup( "<span color=\"green\"><b>%s</b></span>" % path_name_file )
		self.builder.get_object("label_name_dest_project4").set_markup( "<span color=\"green\"><b>%s</b></span>" % path_name_file )

		new_page = self.notebook_main.get_current_page()
		# NOTEBOOK_DEF_PROJECT
		# NOTEBOOK_ADD_LANGUAGES
		# NOTEBOOK_DEF_FOLDERS_PROJECT
		# NOTEBOOK_DEF_SRC_FOLDERS_PROJECT
		# NOTEBOOK_GENERATE

		self.entry_folder_lib.set_text( "lib_" + txt_name_prj )

		if self.ListNameFilesGlade:
			self.checkbutton_folder_pixmaps.set_active( True )
			self.checkbutton_folder_pixmaps.set_sensitive( False )

			self.checkbutton_lib_gtk3.set_active( True )
			self.checkbutton_lib_gtk3.set_sensitive( False )

			self.checkbutton_lib_glib.set_active( True )
			self.checkbutton_lib_glib.set_sensitive( False )

			self.checkbutton_folder_lib.set_active( True )
			self.checkbutton_folder_lib.set_sensitive( False )
			self.entry_folder_lib.set_sensitive( False )
		else:
			self.checkbutton_folder_pixmaps.set_sensitive( True )
			self.checkbutton_lib_gtk3.set_sensitive( True )
			self.checkbutton_lib_glib.set_sensitive( True )
			self.checkbutton_folder_lib.set_sensitive( True )
			self.entry_folder_lib.set_sensitive( True )

		if new_page == NOTEBOOK_DEF_PROJECT:
			if txt_name_prj != "" and txt_version_prj != "" and txt_author_prj != "":
				self.button_next.set_sensitive( True )
			else:
				self.button_next.set_sensitive( False )

		if new_page == NOTEBOOK_ADD_LANGUAGES:
			pass
		if new_page == NOTEBOOK_DEF_FOLDERS_PROJECT:
			pass
		if new_page == NOTEBOOK_DEF_SRC_FOLDERS_PROJECT:
			pass
		if new_page == NOTEBOOK_GENERATE:
			pass

	#
	# Buttons init with notebook_switch_page
	#
	def on_notebook_main_switch_page(self, obj1, obj2, page):
		if page == NOTEBOOK_DEF_PROJECT:
			self.button_previous.hide()
			self.button_next.show()
		elif page == NOTEBOOK_GENERATE:
			self.button_previous.show()
			self.button_next.hide()
		else:
			self.button_previous.show()
			self.button_next.show()
		self.config_buttons_actions()

	def on_button_previous_clicked(self, widget, data=None):
		self.notebook_main.prev_page()
		self.config_buttons_actions()

	def on_button_next_clicked(self, widget, data=None):
		self.notebook_main.next_page()
		self.config_buttons_actions()


	#=========================================
	#
	# PART: NOTEBOOK_DEF_PROJECT
	#
	#=========================================

	#
	# Entry changed
	#
	def on_entry_project_name_changed(self, widget, data=None):
		self.config_buttons_actions()

	def on_entry_project_version_changed(self, widget, data=None):
		self.config_buttons_actions()

	def on_entry_project_author_changed(self, widget, data=None):
		self.config_buttons_actions()


	#
	# List from 'Drag and drop' and FileChooserDialog
	#
	def analyse_list_glade_file( self, p_list_glade_file ):

		codegen = codegen_py()
		several_objects_list = []
		list_from_the_destination_folder = []
		list_name_must_be_different = []

		PathDest = self.button_project_destination.get_label() + "/" + self.entry_project_name.get_text() + "-" + self.entry_project_version.get_text()
		for name_file in p_list_glade_file:
			if os.path.isfile( name_file ):
				if codegen.test_objects_file_glade( name_file ) == True:
					BaseName = os.path.basename( name_file )
					Name = BaseName.split(".")[0]
					Ext  = BaseName.split(".")[-1]
					if Name == self.entry_project_name.get_text():
						if not list_name_must_be_different:
							list_name_must_be_different.append( '<b>' + _('The project name must be different from the file name glade !    ') + '</b>')
						list_name_must_be_different.append( '==> ' + BaseName )
					else:
						if PathDest in name_file:
							if not list_from_the_destination_folder:
								list_from_the_destination_folder.append( '<b>' + _('You can not import a GLADE file from the destination folder !    ') + '</b>')
							list_from_the_destination_folder.append( '==> ' + BaseName )
						elif Ext.lower() == 'glade':
							# test if 'name_file' in liste
							bool_equal = False
							for item in self.ListNameFilesGlade:
								if item == name_file:
									bool_equal = True
									break
							# if ok
							if bool_equal == False:
								# File names can not be identical
								for item in self.ListNameFilesGlade:
									Item_BaseName = os.path.basename( item )
									Item_Name = Item_BaseName.split(".")[0]
									Item_Ext  = Item_BaseName.split(".")[-1]
									if Item_Name == Name:
										bool_equal = True
										break
								# if ok
								if bool_equal == False:
									# add pathname_file in liste
									self.ListNameFilesGlade.append( name_file )
				else:
					if not several_objects_list:
						several_objects_list.append( '<b>' + _('There can be several objects in:    ') + '</b>')
					several_objects_list.append( '==> ' + os.path.basename( name_file ) )
		# Error !
		txt = ""
		for item in several_objects_list:
			txt += '\n' + item
		for item in list_from_the_destination_folder:
			txt += '\n' + item
		for item in list_name_must_be_different:
			txt += '\n' + item
		if txt != '':
			wind_info(
				Form  = self.wind_main,
				Title = _("Warning"),
				Text  = txt
				)
		# put liste
		if not self.ListNameFilesGlade:
			self.on_button_clear_glade_file_clicked( None )
		else:
			txt = ""
			for name_file in self.ListNameFilesGlade:
				BaseName = os.path.basename( name_file )
				Name = BaseName.split(".")[0]
				Ext  = BaseName.split(".")[-1]
				txt = txt + "\n" + Name + "." + Ext
			self.label_drag_and_drop.set_markup( txt )

		self.config_buttons_actions()

	#
	# Select folder to project destination
	#
	def on_button_project_destination_clicked(self, widget, data=None):
		dialog = Gtk.FileChooserDialog(
			_("Please choose a folder"),
			None,
			Gtk.FileChooserAction.SELECT_FOLDER,
			(Gtk.STOCK_CANCEL,
			Gtk.ResponseType.CANCEL,
			_("Select"),
			Gtk.ResponseType.OK))
		dialog.set_current_folder( self.button_project_destination.get_label() )
		dialog.set_default_size(400, 400)
		response = dialog.run()
		if response == Gtk.ResponseType.OK:
			self.button_project_destination.set_label( dialog.get_filename() )
			self.config_buttons_actions()
		dialog.destroy()

	#
	# add glade file from Drag and drop
	#
	def on_drag_data_received( self, widget, drag_context, x,y, data,info, time):
		p_list_glade_file = []
		for uri in data.get_uris():
			# Example uri: file:///home/cat/DEV/BASE_PYTHON/wind_main.glade
			name_file = uri[ 7: ]
			p_list_glade_file.append( name_file )
		self.analyse_list_glade_file( p_list_glade_file )

	#
	# Drag and drop realize
	#
	def on_label_drag_and_drop_realize(self, widget, data=None):
		widget.drag_dest_set(
			Gtk.DestDefaults.MOTION |
			Gtk.DestDefaults.HIGHLIGHT |
			Gtk.DestDefaults.DROP,
			[Gtk.TargetEntry.new("text/uri-list", 0, 80)],
			Gdk.DragAction.COPY,
			)
		widget.connect("drag-data-received", self.on_drag_data_received)

	#
	# add glade file from fileselect
	#
	def on_button_add_glade_file_clicked(self, widget, data=None):
		dialog = Gtk.FileChooserDialog(
			_("Please choose a glade file"),
			None,
			Gtk.FileChooserAction.OPEN,
			(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
			_("Select"),       Gtk.ResponseType.OK))
		dialog.set_current_folder( self.Prefs.Preferences_get_home() )
		dialog.set_default_size(400, 400)
		dialog.set_select_multiple( True )
		response = dialog.run()
		if response == Gtk.ResponseType.OK:
			self.analyse_list_glade_file( dialog.get_filenames() )
		dialog.destroy()

	#
	# clear
	#
	def on_button_clear_glade_file_clicked(self, widget, data=None):
		self.ListNameFilesGlade[:] = []
		self.label_drag_and_drop.set_label( "With glade file: <b>Graphical User Interface</b>\nWith no glade file: <b>Command Line</b>" )
		self.config_buttons_actions()


	#=========================================
	#
	# PART: NOTEBOOK_ADD_LANGUAGES
	#
	#=========================================

	#
	# Add languages and update list
	#
	def Update_languages(self, UpdateList=[]):
		label = ""
		label_1 = ""
		#self.ListPotFile[:] = []
		_ListUpdateLangs = []
		for item in UpdateList:
			label = label + item[1] + "\n"
			label_1 = label_1 + item[2] + "\n"
			#self.ListPotFile.append( item[2] + ".po" )
			_ListUpdateLangs.append( [ item[0], item[1], item[2] ] )
		# if "en.po" in self.ListPotFile:
		# 	self.ListPotFile.append( "en_GB.po" )
		self.ListUpdateLangs[:] = []
		self.ListUpdateLangs = _ListUpdateLangs
		label = label.strip()
		label_1 = label_1.strip()
		self.label_see_languages_1.set_label( label )
		self.label_see_languages_2.set_label( label_1 )
		self.config_buttons_actions()

	def on_button_add_languages_clicked(self, widget, data=None):
		wind_languages(Form=self.wind_main, FuncUpdate=self.Update_languages)


	#=========================================
	#
	# PART: NOTEBOOK_DEF_FOLDERS_PROJECT
	#
	#=========================================

	def get_combobox_folder_menu(self):
		menu = ""
		tree_iter = self.combobox_folder_menu.get_active_iter()
		if tree_iter != None:
			model = self.combobox_folder_menu.get_model()
			menu = model[tree_iter][0]
		return menu

	def on_combobox_folder_menu_realize(self, widget, data=None):
		liststore = Gtk.ListStore(str)
		liststore.append( ["AudioVideo"] )
		liststore.append( ["Audio"] )
		liststore.append( ["Video"] )
		liststore.append( ["Development"] )
		liststore.append( ["Education"] )
		liststore.append( ["Game"] )
		liststore.append( ["Graphics"] )
		liststore.append( ["Network"] )
		liststore.append( ["Office"] )
		liststore.append( ["Settings"] )
		liststore.append( ["System"] )
		liststore.append( ["Utility"] )
		self.combobox_folder_menu.set_model( liststore )
		renderer_text = Gtk.CellRendererText()
		self.combobox_folder_menu.pack_start(renderer_text, True)
		self.combobox_folder_menu.add_attribute(renderer_text, "text", 0)
		value_combo = self.Prefs.Preferences_get_form_combobox_folder_menu_active()
		if value_combo < 0 or value_combo > 11:
			value_combo = 3
		self.combobox_folder_menu.set_active( value_combo )

	def get_combobox_folder_mime(self):
		mime = ""
		tree_iter = self.combobox_folder_mime.get_active_iter()
		if tree_iter != None:
			model = self.combobox_folder_mime.get_model()
			mime = model[tree_iter][0]
		return mime

	def on_combobox_folder_mime_realize(self, widget, data=None):
		liststore = Gtk.ListStore(str)
		liststore.append( ["Text"] )
		liststore.append( ["Audio"] )
		liststore.append( ["Video"] )
		self.combobox_folder_mime.set_model( liststore )
		renderer_text = Gtk.CellRendererText()
		self.combobox_folder_mime.pack_start(renderer_text, True)
		self.combobox_folder_mime.add_attribute(renderer_text, "text", 0)
		value_combo = self.Prefs.Preferences_get_form_combobox_folder_mime_active()
		if value_combo < 0 or value_combo > 2:
			value_combo = 0
		self.combobox_folder_mime.set_active( value_combo )

	def on_checkbutton_folder_clicked(self, widget, data=None):
# 		self.checkbutton_folder_manpage.get_active()
# 		self.checkbutton_folder_pixmaps.get_active()
# 		self.checkbutton_folder_lib.get_active()
# 		self.checkbutton_folder_menu.get_active()
# 		self.checkbutton_folder_mime.get_active()
# 		self.checkbutton_lib_gtk3.get_active()
# 		self.checkbutton_lib_glib.get_active()
		if widget == self.checkbutton_folder_menu:
			if self.checkbutton_folder_menu.get_active():
				self.checkbutton_folder_pixmaps.set_active( True )
		elif widget == self.checkbutton_folder_pixmaps:
			if False == self.checkbutton_folder_pixmaps.get_active() and self.checkbutton_folder_menu.get_active():
				self.checkbutton_folder_menu.set_active( False )

		self.entry_folder_lib.set_sensitive( self.checkbutton_folder_lib.get_active() )
		self.combobox_folder_menu.set_sensitive( self.checkbutton_folder_menu.get_active() )
		self.combobox_folder_mime.set_sensitive( self.checkbutton_folder_mime.get_active() )
		# static_lib <-> glib
		if widget == self.checkbutton_folder_lib:
			if self.checkbutton_folder_lib.get_active():
				self.checkbutton_lib_glib.set_active( True )
		elif widget == self.checkbutton_lib_glib:
			if False == self.checkbutton_lib_glib.get_active() and self.checkbutton_folder_lib.get_active():
				self.checkbutton_folder_lib.set_active( False )





	#=========================================
	#
	# PART: NOTEBOOK_DEF_SRC_FOLDERS_PROJECT
	#
	#=========================================

	def tst_dupply(self):
		bool_true = True
		liste = []
		for row in self.model_ListStore:
			liste.append( row[0] )
		for row in liste:
			if liste.count( row ) > 1:
				bool_true = False
				break;
		self.button_next.set_sensitive( bool_true )

	def _text_changed( self, w, row, new_value, column):
		self.model_ListStore[row][column] = new_value
		self.tst_dupply()

	def on_tree_selection_changed(self, selection, treeview):
		model, rows = treeview.get_selection().get_selected()
		self.selection_ListStore = selection
		if self.selection_ListStore:
			model, treeiter = self.selection_ListStore.get_selected()
			if treeiter != None:
				if model[treeiter][2] != False:
					self.button_remove_folder_src.set_sensitive( True )
				else:
					self.button_remove_folder_src.set_sensitive( False )
			else:
				if rows != None:
					self.button_remove_folder_src.set_sensitive( True )
				else:
					self.button_remove_folder_src.set_sensitive( False )
		self.tst_dupply()

	def on_viewport_src_folders_realize(self, widget, data=None):
		# Def line with ListStore
		self.model_ListStore = Gtk.ListStore( str, str, bool, str, str, bool, str, str, bool )
		self.model_ListStore.append( [ "src", "#FFFFFF", False, self.entry_project_name.get_text().strip(), "#FFFFFF", True, "test", "#FFFFFF", False ])
		# Def treeview
		treeview = Gtk.TreeView( self.model_ListStore )

		# Def title column text
		col_src = Gtk.TreeViewColumn( "Src")
		treeview.append_column( col_src )
		# Def cell
		cell = Gtk.CellRendererText()
		col_src.pack_start( cell, expand=False)
		# Def attributes
		col_src.set_attributes( cell, text=0, cell_background=1, editable=2)
		col_src.set_fixed_width( 100 );
		cell.connect('edited', self._text_changed, 0)

		# Def title column text
		col_exec = Gtk.TreeViewColumn( "Exec")
		treeview.append_column( col_exec )
		# Def cell
		cell = Gtk.CellRendererText()
		col_exec.pack_start( cell, expand=False)
		# Def attributes
		col_exec.set_attributes( cell, text=3, cell_background=4, editable=5)
		col_exec.set_fixed_width( 100 );
		cell.connect('edited', self._text_changed, 3)


		# Def title column text
		col_test = Gtk.TreeViewColumn( "Test")
		treeview.append_column( col_test )
		# Def cell
		cell = Gtk.CellRendererText()
		col_test.pack_start( cell, expand=False)
		# Def attributes
		col_test.set_attributes( cell, text=6, cell_background=4, editable=8)
		cell.connect('edited', self._text_changed, 6)

		#
		select = treeview.get_selection()
		select.connect("changed", self.on_tree_selection_changed, treeview)
		# Line select
		treeview.set_cursor(0)
		#
		widget.add(treeview)
		widget.show_all()

	def on_button_add_folder_src_clicked(self, widget, data=None):
		_src = "src"
		_num = 1
		bool_cont = False
		while True:
			for row in self.model_ListStore:
				if row[0] == _src:
					_src = "src_" + str( _num )
					bool_cont = True
					break
				else:
					bool_cont = False
			if bool_cont == False:
				break
			_num += 1

		self.model_ListStore.append( [
			_src,
			"#FFFFFF",
			True,
			self.entry_project_name.get_text().strip(),
			"#FFFFFF",
			True,
			"Test",
			"#FFFFFF",
			False
			] )
		self.tst_dupply()

	def get_list_store(self):
		list_store = []
		for row in self.model_ListStore:
			list_store.append( [ row[0], row[3] ] )
		return list_store

	def on_button_remove_folder_src_clicked(self, widget, data=None):
		if self.selection_ListStore:
			model, treeiter = self.selection_ListStore.get_selected()
			if treeiter != None:
				if model[treeiter][2] != False:
					#print( "Selected is", model[treeiter][:] )
					self.model_ListStore.remove( treeiter )
		self.tst_dupply()


# gtkliststore editable python 3
# http://python.developpez.com/cours/pygtktutorial/php/pygtkfr/sec-CellRenderers.php
# http://python.zirael.org/e-gtk-treeview3.html


	#=========================================
	#
	# PART: NOTEBOOK_GENERATE
	#
	#=========================================

	def timeout_create_project(self, user_data):
		self.progressbar_main.pulse()
		# upper  lower
		adj = self.scrolledwindow_see_project_generate.get_vadjustment()
		adj.set_value( adj.get_property('upper') )
		self.scrolledwindow_see_project_generate.set_vadjustment( adj )
		if self.timeout_id > 0:
			return True
		else:
			self.box_previous_next.show()
			self.box_progressbar.hide()
			self.button_generate.set_sensitive( True )
			return False

	def set_mess( self, text ):
		old_txt = ''
		old_txt = self.label_see_project_generate.get_label()
		old_txt += '\n' + text
		self.label_see_project_generate.set_markup( old_txt.strip() )

	def create_project(self, PathDest):
		self.label_see_project_generate.set_label( '' )
		self.set_mess( 'CREATION PROJECT:\n\t%s' % os.path.basename( PathDest ))
		self.set_mess( '\nCREATE DIRECTORY:\n\t%s' % PathDest )
		# init build_write( pathdest="", list_po=[], bool_manpage=False, bool_pixmaps=False, name_lib="", name_menu="", name_mime="" )
		name_lib_statique = ""
		if self.checkbutton_folder_lib.get_active() == True:
			name_lib_statique = self.entry_folder_lib.get_text()
		name_menu = ""
		if self.checkbutton_folder_menu.get_active() == True:
			name_menu = self.get_combobox_folder_menu()
		name_mime = ""
		if self.checkbutton_folder_mime.get_active() == True:
			name_mime = self.get_combobox_folder_mime()
		build = build_write(
			project_name=self.entry_project_name.get_text(),
			project_version=self.entry_project_version.get_text(),
			project_author=self.entry_project_author.get_text(),
			pathdest=PathDest,
			list_po=self.ListUpdateLangs,
			bool_manpage=self.checkbutton_folder_manpage.get_active(),
			bool_pixmaps=self.checkbutton_folder_pixmaps.get_active(),
			name_lib_statique=name_lib_statique,
			name_menu=name_menu,
			name_mime=name_mime,
			bool_gtk3=self.checkbutton_lib_gtk3.get_active(),
			bool_glib=self.checkbutton_lib_glib.get_active(),
			list_files_glade=self.ListNameFilesGlade,
			list_src=self.get_list_store()
			 )

#		CREATION FICHIER Makefile.am
		build.write_makefile_am()

# 		LES DOSSIERS LIBRAIRIE STATIQUE
		build.write_lib_statique()
		if name_lib_statique:
			self.set_mess( '\t%s/%s' % (PathDest, name_lib_statique))

# 		CREATE DOSSIER GLADE
		build.write_glade_makefile_am()
		if self.ListNameFilesGlade:
			self.set_mess( '\t%s/glade' % PathDest )

# 		CREATE DOSSIER SRC
		build.write_folder_src()
		list_src = self.get_list_store()
		for ls_src in list_src:
			self.set_mess( '\t%s/%s' % (PathDest, ls_src[0]))

# 		CREATE DOSSIER MANPAGE
		build.write_folder_manpage()
		if self.checkbutton_folder_manpage.get_active():
			self.set_mess( '\t%s/man' % PathDest )

# 		CREATE DOSSIER PIXMAPS + icone.png si bool_gtk
		build.write_folder_pixmaps()
		if self.checkbutton_folder_pixmaps.get_active():
			self.set_mess( '\t%s/pixmaps' % PathDest )

# 		CREATE DOSSIER MENU
		build.write_folder_menu()
		if name_menu != '':
			self.set_mess( '\t%s/menu' % PathDest )

# 		CREATE DOSSIER MIME
		build.write_folder_mime()
		if name_mime != '':
			self.set_mess( '\t%s/mime' % PathDest )

# 		SE PLACER DANS LE DOSSIER VarBuild.NewPathName/
# 		CREATION FICHIER configure.in
		build.write_configure_ac()

# 		MODIFICATION DOSSIER PO
# 		CREATE DOSSIER PO
		build.create_folder_po()
		if self.ListUpdateLangs:
			self.set_mess( '\t%s/po' % PathDest )
# 		RE CREATION FICHIER configure.ac
		build.write_configure_ac()
# 		RE CREATION FICHIER Makefile.am
		build.write_makefile_am()

# 		CREATION FICHIERS ./autogen.sh
# 		CREATION FICHIERS  ./TODO ./NEWS ./README ./AUTHORS ./ChangeLog
		build.write_autogen_sh()

# 		CONSTRUCTION DU PAQUET
		os.chdir( PathDest )
		self.set_mess( '\nACTIONS:' )
		self.set_mess( '\tautoscan' )
		subprocess.call( "autoscan", shell=True )
		self.set_mess( '\taclocal' )
		subprocess.call( "aclocal", shell=True )
		self.set_mess( '\tautoheader' )
		subprocess.call( "autoheader", shell=True )
		self.set_mess( '\tautoconf' )
		subprocess.call( "autoconf", shell=True )
		self.set_mess( '\tautomake -a -c' )
		subprocess.call( "automake -a -c", shell=True )
		self.set_mess( '\tautomake --add-missing' )
		subprocess.call( "automake --add-missing", shell=True )
		self.set_mess( '\tautomake' )
		subprocess.call( "automake", shell=True )
		self.set_mess( '\t./configure --prefix=/usr' )
		subprocess.call( "./configure --prefix=/usr", shell=True )
		self.set_mess( '\tmake' )
		subprocess.call( "make", shell=True )
		self.set_mess( '\tmake distcheck' )
		subprocess.call( "make distcheck", shell=True )
		if os.path.isdir( PathDest + '/' + self.entry_project_name.get_text() + '-' + self.entry_project_version.get_text() ) == True:
			self.set_mess( '\n<b>*---------*</b>' )
			mess = _('Error from') + ' distcheck'
			self.set_mess( mess )
			self.set_mess( '<b>*---------*</b>' )
		if os.path.isfile( '%s/autoscan.log' % PathDest ):
			self.set_mess( '\n<b>*---------*</b>' )
			mess = _('This file') + '\n\t[ %s/configure.ac ]\n' % PathDest + _('must be Completed since this file') + '\n\t[ %s/autoscan.log ]' % PathDest
			self.set_mess( mess )
			self.set_mess( '<b>*---------*</b>' )

# 		SE REPLACER DANS LE DOSSIER D ORIGINE
		self.timeout_id = -1

	def on_button_generate_clicked(self, widget, data=None):
#		TEST PRESENCE DES PROGRAMMES EXTERNES
		TxtErr = ""
		if os.path.isfile( "/usr/bin/gettextize" ) == False and self.ListUpdateLangs:
			TxtErr = "gettextize\n"
		if os.path.isfile( "/usr/bin/autoscan" ) == False:
			TxtErr += "autoscan\n"
		if os.path.isfile( "/usr/bin/aclocal" ) == False:
			TxtErr += "aclocal\n"
		if os.path.isfile( "/usr/bin/autoheader" ) == False:
			TxtErr += "autoheader\n"
		if os.path.isfile( "/usr/bin/autoconf" ) == False:
			TxtErr += "autoconf\n"
		if os.path.isfile( "/usr/bin/automake" ) == False:
			TxtErr += "automake\n"
		if os.path.isfile( "/usr/bin/make" ) == False:
			TxtErr += "make\n"
		if TxtErr != "":
			TxtErr = "\n<b>" + _("Please install these programs:") + "</b>\n\n" + TxtErr
			wind_info(
				Form  = self.wind_main,
				Title = _("Programs not found"),
				Text  = TxtErr )
			return

# 		SUPPRESSION DU DOSSIER SI IL EXISTE
		txt_name_prj = self.entry_project_name.get_text().strip()
		txt_version_prj = self.entry_project_version.get_text().strip()
		PathDest = self.button_project_destination.get_label() + "/" + txt_name_prj + "-" + txt_version_prj
		if os.path.isdir( PathDest ) == True:
			dialogWindow = Gtk.MessageDialog(None,
								  Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
								  Gtk.MessageType.QUESTION,
								  Gtk.ButtonsType.OK_CANCEL,
								  _("This folder exist.\nDo you want to replace this folder ?") )
			response = dialogWindow.run()
			dialogWindow.destroy()
			while Gtk.events_pending():
				Gtk.main_iteration()
			if response == Gtk.ResponseType.CANCEL:
				return
			if response == Gtk.ResponseType.OK:
				# remove directory
				command = "chmod -R 777  " + PathDest
				subprocess.call( command, shell=True )
				command = "rm -rf " + PathDest
				subprocess.call( command, shell=True )

# 		CREATION DOSSIER DU PROJET
		os.makedirs( PathDest )

#		THREAD
		self.button_generate.set_sensitive( False )
		self.box_previous_next.hide()
		self.box_progressbar.show()
		self.label_progressbar.set_label( _("Please wait while the treatment") )
		self.timeout_id = GObject.timeout_add( 100, self.timeout_create_project, None )
		thread = threading.Thread( target=self.create_project, args=(PathDest,))
		thread.start()


if __name__ == '__main__':
	wind = wind_main()
	Gtk.main()

